import sys

from javax.swing import JPopupMenu, JFrame, JMenuBar, JMenu, JMenuItem, JTextField, JFileChooser, JLabel, JPanel, JSplitPane, JButton
from javax.swing.border import LineBorder
from java.awt import GridLayout, BorderLayout, FlowLayout, Color, Dimension, Canvas, Toolkit, Font
from java.awt.event import InputEvent
from java.awt.datatransfer import DataFlavor
from java.lang import System, Runtime, Float

from visad import *
from visad.bom import HydraImageRendererJ3D
import subs
import graph
from subs import *
from visad.python.JPythonMethods import *
from ui.colorbar import ColorBarWidget, ColorBar

import MultiSpectralDataSource
import jarray
from hframe import HFrame
from subsetImage import SubsetImage, Subset
from histostretch import linkHistLocation, dragHistLocation

from util import Helper
from html_hydra import HtmlBrowserWindow

import hydra_properties
hydra_properties.setProperties()

import hydra_globals
from hydra_globals import hydra_path

from roamProbe import roamProbe

false = 0
true  = 1

runtime=Runtime.getRuntime()
subsetRef=[]
subsetRef.append(None)

def doChangeMarker(x,y):
  global shapes, shapeMarkerIndex
  shapes.moveShape(shapeMarkerIndex,(x,y))

def windowClose(event):
    newFrame.dispose()
    runtime.gc()

def makeMod04Display(displayData, fileName=None):
     global newDisp,maps,img,newFrame, shapes, shapeMarkerIndex
     
     obsElementInstrument, obsLineInstrument, obsChannelIndexInstrument, observationChannelsInstrument, observationsInstrument = displayData.getRealTypes()
     obsLatitudeInstrument, obsLongitudeInstrument  = displayData.getGeoRealTypes()
     txtType = TextType.Generic
  
     latminmax, lonminmax = displayData.getGeoDomain()
  
     londiff = lonminmax[1] - lonminmax[0]
     latdiff = latminmax[1] - latminmax[0]
     if londiff >= latdiff:
      lon_low = lonminmax[0]
      lon_hi  = lonminmax[1]
      lat_low = latminmax[0]
      lat_hi  = lat_low + londiff
     else:
      lat_low = latminmax[0]
      lat_hi  = latminmax[1]
      lon_low = lonminmax[0]
      lon_hi  = lon_low + latdiff

     maps = subs.makeMaps(obsLongitudeInstrument,'x', obsLatitudeInstrument, 'y', observationsInstrument, 'rgb', txtType, 'text', 'marker', 'shape')
     
     # open a new window if opening Level 2 Products
     newFrame  = HFrame("Hydra MODIS Aerosol Product",windowClosing=windowClose)
     menuBar   = JMenuBar()
     newFrame.setJMenuBar(menuBar)
     
     settingsMenu = JMenu("Settings")
     global colorRangeItem
     colorRangeItem = JMenuItem("Set Color Range", actionPerformed=setColorScale)
     colorRangeItem.setEnabled(1)
     settingsMenu.add(colorRangeItem)
     menuBar.add(settingsMenu)
        
     toolMenu = JMenu("Tools")
     toolItem = JMenuItem("Capture Display", actionPerformed=captureDisplay)
     toolItem.setEnabled(1)
     toolMenu.add(toolItem)
     menuBar.add(toolMenu)
     
     newDisp = makeDisplay(maps)
     newDspRenderer = newDisp.getDisplayRenderer()
          
     newDspRenderer.setBackgroundColor(hydra_properties.background_color)
     newDspRenderer.setForegroundColor(hydra_properties.foreground_color)
     splitPane=JSplitPane(JSplitPane.HORIZONTAL_SPLIT)
     framePane = newFrame.getContentPane()
     framePane.setLayout(BorderLayout())
     #framePane.add(disp.getComponent(),BorderLayout.WEST)
     splitPane.add(newDisp.getComponent())
     splitPane.setContinuousLayout(1)
     splitPane.setOneTouchExpandable(1)
     splitPane.setDividerSize(12)
     framePane.add("Center",splitPane)
        
     from DisplayViewSelect import DisplayViewSelectToolBar, RubberBandSubset
     toolbar = DisplayViewSelectToolBar([newDisp])
     toolbar.addHAction(RubberBandSubset(toolbar.listeners, subsetRef, icon="./ui/icons/subset16.jpg", tooltipText="Subset Image"))
     framePane.add("North", toolbar)
                  
     map=load('./data/outlsupw')
     #subs.addData('supw',map,newDisp,constantMaps=makeColorMap(hydra_properties.foreground_color))
     newDisp.addData('supw',map,constantMaps=makeColorMap(hydra_properties.foreground_color))
     #newDisp.getGraphicsModeControl().setMissingTransparent(true)
     #newDisp.addMaps(maps)
     
     newDisp.disableAction()
     channel_idx = 0
     
     londiff = lonminmax[1] - lonminmax[0]
     latdiff = latminmax[1] - latminmax[0]
     if londiff >= latdiff:
        lon_low = lonminmax[0]
        lon_hi  = lonminmax[1]
        lat_low = latminmax[0]
        lat_hi  = lat_low + londiff
     else:
        lat_low = latminmax[0]
        lat_hi  = latminmax[1]
        lon_low = lonminmax[0]
        lon_hi  = lon_low + latdiff
          
     maps[0].setRange(lon_low, lon_hi)
     maps[1].setRange(lat_low, lat_hi)  
          
     if displayData.numObsElements < 512:
        img = displayData.getImageSegment(channel_idx,0)
     else:
        img = displayData.getImageSegment(channel_idx,0, Subset(None,None,None,None,step=10))
          
     ir=HydraImageRendererJ3D()
     ir.setMissingColor(255,255,255) # set missing to white
     imageRef=subs.addData('a',img,newDisp,renderer=ir, zlayer = -0.1)
     subs.addData('b',map,newDisp,constantMaps=makeColorMap(hydra_properties.foreground_color))
     si = SubsetImage(img,newDisp,maps[0],maps[1],changeSubset,subsetRef,color="green")
          
     colorBar = ColorBarWidget(maps[2],hydra_properties.background_color, hydra_properties.foreground_color)
     colorBar.setPreferredSize(Dimension(20,400))
     splitPane.add(colorBar)

     txtCntrl = maps[3].getControl()
     txtCntrl.setJustification(TextControl.Justification.LEFT)
     txtCntrl.setSize(1.5)
     txtCntrl.setAutoSize(1)
     txtCntrl.setFont(Font("sansserif", Font.PLAIN, 14))

     shapes = subs.Shapes(newDisp,maps[4])
     shapeMarkerIndex = shapes.addShape("cross",scale=.03,color="cyan", autoScale=0)
     shapeRef = shapes.shapeRef[shapeMarkerIndex]


     roamText = Tuple(TupleType([obsLatitudeInstrument, obsLongitudeInstrument, txtType]), [Real(obsLatitudeInstrument, Float.NaN), Real(obsLongitudeInstrument, Float.NaN), Text(txtType, '   %-6.2f' % Float.NaN)])
     roamTextRef = subs.addData('roamText', roamText, newDisp, subs.makeColorMap(Color(1.0,0.0,1.0)),zlayer = 0.1)
     r_probe = roamProbe(newDisp, displayData, toolbar, roamTextRef, obsLatitudeInstrument, obsLongitudeInstrument, txtType, displayData.getImageValue)

     subs.HandlePickEvent( newDisp, doChangeMarker)
     
     newFrame.pack()
     newFrame.setSize(500,500)
     newFrame.show()
     newDisp.enableAction()

def setColorScale(event):
  dragHistLocation(maps[2], img)
  
def captureDisplay(event):
  Helper.saveDisplay(newDisp, JFrame().getContentPane())

def changeSubset(a,b):
    pass
  
class AerosolData:
      def __init__(self, dataSource):
        self.dataSource = dataSource
        self.rgbMap = None
        obsElementInstrument, obsLineInstrument, obsChannelIndexInstrument, observationChannelsInstrument, observationsInstrument = displayData.getRealTypes()
        self.rgbRealType = observationsInstrument
        self.dataReference = DataReference('aerosolData')
        self.positionRef = None

        self.init()

      def init(self):
        if displayData.numObsElements < 512:
          img = displayData.getImageSegment(channel_idx,0)
        else:
          img = displayData.getImageSegment(channel_idx,0, Subset(None,None,None,None,step=10))

        self.dataReference.setData(img)

      def getDataReference(self):
        return self.dataReference

      def getRGBRealType(self):
        return self.rgbRealType

      def getPositionRef(self):
        return self.positionRef
