import sys

from javax.swing import JPopupMenu, JFrame, JMenuBar, JMenu, JMenuItem, JTextField, JFileChooser, JLabel, JPanel, JSplitPane, JButton, JRadioButtonMenuItem, ButtonGroup
from javax.swing.border import LineBorder
from java.awt import GridLayout, BorderLayout, FlowLayout, Color, Dimension, Canvas, Toolkit, Font
from java.awt.event import InputEvent, ActionEvent
from java.awt.datatransfer import DataFlavor
from javax.swing.border import LineBorder, CompoundBorder, EmptyBorder
from java.lang import System, Float, Runtime, Math, Double, String

import visad
from visad import *
from visad.java3d import DirectManipulationRendererJ3D
from visad.bom import HydraImageRendererJ3D
from visad import CommonUnit
import mapProjections
import subs
import graph
from subs import *
from visad.python.JPythonMethods import *
from ui.colorbar import ColorBarWidget, ColorBar

import MultiSpectralDataSource
import jarray
from hframe import HFrame
from subsetImage import SubsetImage, Subset
from histostretch import linkHistLocation, dragHistLocation

from util import Helper, ScalarMapSyncher
from util import LambertEA, SatProjection, MapProjectionAdapter
from html_hydra import HtmlBrowserWindow

import hydra_properties
from hydra_properties import true, false
hydra_properties.setProperties()
runtime = Runtime.getRuntime()

global adaptProj
import hydra_globals
from roamProbe import roamProbe
from hydra_globals import geoPointRef_s, hydra_path

subsetRef=[]
subsetRef.append(None)

import ColorScaleSelect
from ColorScaleSelect import ColorScaleSelect

import transect


class airsLevel2Source:
  def __init__(self, displayData, subset=None):
    self.displayData = displayData
    self.lastLevelIndx = -1
    self.lastRefMarker = None
    self.refMarkerRef = None
    self.refSpectrumRef = None
    self.currentProj = "Lambert Equal Area"
    self.currentLine  = 0
    self.currentElem  = 0
    self.diffMap = None
    self.r_probe = None
    self.colorBar = None
    self.imageRef = None
    self.refProfileRef = None
    self.h_action = None
    self.colorBarOn = false

    #global airsLevel2Disp, airsLevel2Frame,map,splitPane,levelMenu
     
    # open a new window if opening Level 2 Products
    self.frame  = HFrame("Hydra AIRS Level 2 Products",windowClosing=self.windowClose)
    menuBar   = JMenuBar()
    self.frame.setJMenuBar(menuBar)
     
    varMenu = JMenu("Variables")
     
    varsToAdd=displayData.imageRangeTypeDict.keys()
    for i in xrange(displayData.numObsDataSets):
         varMenuItem=JMenuItem(varsToAdd[i],actionPerformed=self.changeVariable)
         varMenuItem.setEnabled(1)
         varMenu.add(varMenuItem)
             
    menuBar.add(varMenu)
     
    self.levelMenu=JMenu("Levels")

   
    levelsToAdd=displayData.pressureLevels
    menuLevels = [1100,1050,1000,950,900,850,800,700,600,500,400,300,200,250,200,100,50]
    gset = Gridded1DSet(RealType.Generic, [displayData.pressureLevels], len(displayData.pressureLevels))
    for j in xrange(len(menuLevels)):
      idx = gset.valueToIndex([[menuLevels[j]],])[0]
      if idx >=0 and idx < len(displayData.pressureLevels):
       levMenuItem=JMenuItem(levelsToAdd[idx].toString(),actionPerformed=self.changeLevel)
       levMenuItem.setEnabled(1)
       self.levelMenu.add(levMenuItem)

    
    if self.displayData.isProfile:
      self.levelMenu.setEnabled(1)
    else:
      self.levelMenu.setEnabled(0)
      
    menuBar.add(self.levelMenu)
          
    settingsMenu = JMenu("Settings")
    global colorRangeItem
    colorRangeItem = JMenuItem("Set Color Range", actionPerformed=self.setColorScale)
    colorRangeItem.setEnabled(1)
    settingsMenu.add(colorRangeItem)
    self.colorMenu = ColorScaleSelect("Set Color Scale", None, "Color")
    settingsMenu.add(self.colorMenu)
    menuBar.add(settingsMenu)
    menu = JMenu("Show ColorBar")
    bg = ButtonGroup()
    togButton = JRadioButtonMenuItem("On", actionPerformed=self.toggleColorBar)
    togButton.setSelected(0)
    bg.add(togButton)
    menu.add(togButton)
    togButton = JRadioButtonMenuItem("Off", actionPerformed=self.toggleColorBar)
    togButton.setSelected(1)
    bg.add(togButton)
    menu.add(togButton)
    settingsMenu.add(menu)
        
    toolMenu = JMenu("Tools")
    toolItem = JMenuItem("Capture Display", actionPerformed=self.captureDisplay)
    toolItem.setEnabled(1)
    toolMenu.add(toolItem)
    item4 = JMenuItem("Transect", actionPerformed=self.doTransect)
    item4.setEnabled(1)
    toolMenu.add(item4)
    menu = JMenu("Reference Profile")
    self.refProfToggleButton = JRadioButtonMenuItem("On", actionPerformed=self.refProfileToggle)
    self.refProfToggleButton.setSelected(0)
    menu.add(self.refProfToggleButton)
    self.profDiffToggleButton = JRadioButtonMenuItem("diff", actionPerformed=self.profileDiffToggle)
    self.profDiffToggleButton.setSelected(0)
    menu.add(self.profDiffToggleButton)
    toolMenu.add(menu)


    menuBar.add(toolMenu)
       
    # make dummy maps and displays
    self.airsLevel2Map=makeMaps(RealType.Longitude, "x", RealType.Latitude, "y")
    self.airsLevel2Disp=makeDisplay(self.airsLevel2Map)
    ##self.profileMap=makeMaps(self.displayData.imageRangeType,"x",self.displayData.pressureType, "y")
    self.profileMap=makeMaps(self.displayData.imageRangeType,"x",self.displayData.logPressType, "y")
    from java.util import Hashtable
    logLabels = Hashtable()
    logLabels.put(Double(1.8), '60')
    logLabels.put(Double(2.1), '125')
    logLabels.put(Double(2.4), '250')
    logLabels.put(Double(2.7), '500')
    logLabels.put(Double(3.0), '1000')
    pAxisScale = self.profileMap[1].getAxisScale()
    pAxisScale.setLabelTable(logLabels)
    pAxisScale.setMajorTickSpacing(0.3)

    self.profileMap[1].setRange(3.0, 1.8)

    self.profileDisp=makeDisplay(self.profileMap)

    validRange=self.displayData.getObsValidRange()
    self.rrbzRef = self.profileDisp.enableRubberBandBoxZoomer(1)    
        
    airsLevel2DspRenderer = self.airsLevel2Disp.getDisplayRenderer()
    airsLevel2DspRenderer.setBackgroundColor(hydra_properties.background_color)
    airsLevel2DspRenderer.setForegroundColor(hydra_properties.foreground_color)
    
    framePane = self.frame.getContentPane()
    framePane.setLayout(BorderLayout())

    #-new TDR
    #set up profile panel
    self.profilePane=JPanel()
    self.profilePane.setLayout(BorderLayout())

    self.imagePanel = JPanel(BorderLayout())
    self.splitPane=JSplitPane(JSplitPane.VERTICAL_SPLIT)
    self.splitPane.setContinuousLayout(1)
    self.splitPane.setOneTouchExpandable(1)
    self.splitPane.setDividerSize(12)
    #---------------------------------------------

    
    
    from DisplayViewSelect import DisplayViewSelectToolBar, RubberBandSubset
    toolbar = DisplayViewSelectToolBar([self.airsLevel2Disp,self.profileDisp],[0, 1])
    
    self.toolbar=toolbar
    self.text = TextType.Generic              
    self.map=load('./data/outlsupw')
         
    self.rebuildAIRSLevel2Display()

    
    
    self.mapRef = subs.addData('m',self.map,self.airsLevel2Disp,constantMaps=subs.makeColorMap(hydra_properties.foreground_color)+[ConstantMap(-0.1, self.displayAltType)])

    # enable the Listener for mouse up events
    subs.HandlePickEvent(self.airsLevel2Disp, self.doChangeProfile)

    self.southPanel = JPanel(GridLayout(2,1,0,0))
    self.imagePanel.add(self.airsLevel2Disp.getComponent(), "Center")
    self.splitPane.add(self.profilePane)
    self.splitPane.add(self.imagePanel)
    
    framePane.add("Center", self.splitPane)
    framePane.add("South", toolbar)
    self.toolbar = toolbar
    self.framePane = framePane

    self.frame.pack()
    self.frame.show()
    if self.displayData.isProfile:
      self.frame.setSize(500,700)
    else:
      self.frame.setSize(600,500)

 
    ##self.addImageText()
    
    class DspListnr(DisplayListener):
      def __init__(self, parent):
        self.dsp = parent.airsLevel2Disp
        self.dsp.addDisplayListener(self)
        self.refMarkerRef = parent.refMarkerRef
        self.last = self.refMarkerRef.getData()
        self.parent = parent
                                                                                                                                
      def displayChanged(self, event):
        if event.getId() == 7:
          marker = self.parent.refMarkerRef.getData()
          vals = self.parent.refMarkerRef.getData().getValues()
          last_vals = self.last.getValues()
          if (visad.util.Util.isApproximatelyEqual(vals[0],last_vals[0]) and visad.util.Util.isApproximatelyEqual(vals[1],last_vals[1])):
            return 
          else:
            lat  = vals[0]
            lon  = vals[1]
            self.parent.updateGeoPoint(lon, lat, 1)
            self.last = marker
    self.refProfDispListnr = DspListnr(self)


    from lockCell import LockCell
    from util import Syncher
    self.geoPtCell = LockCell(geoPointRef_s[0], self.updateProfile)
    self.geoPtCell.addReference(geoPointRef_s[0])
    self.geoPtCell_1 = LockCell(geoPointRef_s[1], self.updateRefProfile)
    self.geoPtCell_1.addReference(geoPointRef_s[1])

    self.ad = self.displayData
    self.refToImage = self.imageRef

     
  def rebuildAIRSLevel2Display(self):
     # clear display before creating a new one
     self.airsLevel2Disp.removeAllReferences()
     self.airsLevel2Disp.clearMaps()
     
     self.obsLatitudeInstrument, self.obsLongitudeInstrument  = self.displayData.getGeoRealTypes()
     
     if self.displayData.numObsElements < 512:
        img = self.displayData.getImageSegment(0)
     else:
        img = self.displayData.getImageSegment(0, Subset(None,None,None,None,step=10))
     
     self.observationsInstrument = self.displayData.imageRangeType

     rt = geoPointRef_s[0].getData()
     if rt != None:
       vals = rt.getValues()
       lat = vals[1]
       lon = vals[0]
       le = self.displayData.cs.fromReference([[lat],[lon]])
       le = self.imageToSubset(le[0][0], le[1][0])
       self.currentLine = le[0]
       self.currentElem = le[1]

     
     # set up the profile display (if necessary)
     self.profileDisp.removeAllReferences()
     if self.displayData.isProfile:
       self.profile = self.displayData.getObservations(self.currentLine,self.currentElem)
       self.profileDisp.removeMap(self.profileMap[0])
       map = subs.makeMaps(self.observationsInstrument,"x")
       self.profileMap[0] = map[0]
       validRange = self.displayData.getObsValidRange()
       self.pressureRange = self.displayData.getPressureRange() # will not change
       self.profileMap[0].setRange(validRange[0],validRange[1])
       self.profileMap[1].setRange(3.0,1.8)
       self.profileDisp.addMap(self.profileMap[0])
       dsp_rdr = self.profileDisp.getDisplayRenderer()
       dsp_rdr.setBackgroundColor(hydra_properties.background_color)
       dsp_rdr.setForegroundColor(hydra_properties.foreground_color)
       self.profileRef = subs.addData('b',self.profile,self.profileDisp,subs.makeColorMap(hydra_properties.foreground_color))
       
       # add a rubber band box and set the axes properties
       self.rrbzRef = self.profileDisp.enableRubberBandBoxZoomer(1)
       subs.setPointSize(self.profileDisp, 4)
       showAxesScales(self.profileDisp,1)
       subs.setAspectRatio(self.profileDisp,2.2)
       subs.setBoxSize(self.profileDisp,.6,clip=0)
       # draw a horizontal green line to indicate the current level
       self.x0 = validRange[0]
       self.x1 = validRange[1]
       self.profileDisp.disableAction()
       pressValue = self.displayData.pressureLevels[self.displayData.levelValue]
       lineValue = Math.log(pressValue)/Math.log(10)
       self.lineRef = subs.drawLine(self.profileDisp,[(self.x0,self.x1),(lineValue,lineValue)],color="green")

       if self.refProfileRef == None:
         rt = geoPointRef_s[1].getData()
         lat = self.displayData.obsMin[0]
         lon = self.displayData.obsMin[1]
         if rt != None:
           vals = rt.getValues()
           lat = vals[1]
           lon = vals[0]
         le = self.displayData.cs.fromReference([[lat],[lon]])
         le = self.imageToSubset(le[0][0], le[1][0])
         self.refLine = le[0]
         self.refElem = le[1]
         refProfile = self.displayData.getObservations(self.refLine, self.refElem, true)
         self.refProfileRef = subs.addData('d',refProfile,self.profileDisp, subs.makeColorMap('magenta'))
         self.profileDisp.toggle(self.refProfileRef, false)
       else:
         refProfile = self.displayData.getObservations(self.refLine, self.refElem, true)
         self.refProfileRef.setData(refProfile)
         self.profileDisp.addReference(self.refProfileRef, subs.makeColorMap('magenta'))
         self.profileDisp.toggle(self.refProfileRef, false)
         if self.h_action != None:
           self.toolbar.removeHAction(self.h_action)
       self.profDiffToggleButton.setSelected(0)
       self.initProfileDiff()
       self.updateProfileDiff()

       self.profileDisp.enableAction()

       subs.HandlePickEvent(self.profileDisp, self.changeLevelImage)
       self.profilePane.add("Center",self.profileDisp.getComponent())
     else:
       # blot out profile map so it is not visible
       dsp_rdr = self.profileDisp.getDisplayRenderer()
       from java.awt import Color
       dsp_rdr.setForegroundColor(Color.lightGray)
       dsp_rdr.setBackgroundColor(Color.lightGray)
              
     corners = mapProjections.getValidCorners(self.displayData.gs, [[-180,180], [-90, 90]])
     
     minLon = min(corners[0])
     minLat = min(corners[1])
     maxLon = max(corners[0])
     maxLat = max(corners[1])
     minPres = min(self.displayData.pressureLevels)
     maxPres = max(self.displayData.pressureLevels)

     displayLatType   = DisplayRealType("ProjectionLat_%d" % hydra_globals.dtuple_cnt, true, minLat, maxLat, 0.0, CommonUnit.degree)
     displayLonType   = DisplayRealType("ProjectionLon_%d" % hydra_globals.dtuple_cnt, true, minLon, maxLon, 0.0, CommonUnit.degree)
     displayAltType   = DisplayRealType("ProjectionAlt_%d" % hydra_globals.dtuple_cnt, true, -1.0, 1.0, 0.0, None)

     adaptProj = MapProjectionAdapter(LambertEA(corners))
     
     self.ds_cs = adaptProj
     
     displayTupleType = DisplayTupleType([displayLonType, displayLatType, displayAltType], adaptProj)

     self.maps = subs.makeMaps(self.obsLongitudeInstrument,'x', self.obsLatitudeInstrument, 'y', self.observationsInstrument, 'rgb',
                'marker','shape', self.text, 'text', RealType.XAxis, 'x', RealType.YAxis, 'y')
     #- replace first two with user-defined display types
     self.maps[0] = ScalarMap(self.obsLongitudeInstrument, displayLonType)
     self.maps[1] = ScalarMap(self.obsLatitudeInstrument, displayLatType)
     self.maps[5].setRange(-1, 1)
     self.maps[6].setRange(-1, 1)
     self.maps = self.maps+[ScalarMap(RealType.Altitude, displayAltType)]
     self.altitudeMap = self.maps[7]
     self.rgbMap = self.maps[2]
     
     self.airsLevel2Disp.addMaps(self.maps)
     
     # get the dimensions
     dimInstrumentDomain = img.getDomainSet()
     self.dimInstrument = dimInstrumentDomain.getLengths()
     
     self.airsLevel2Disp.disableAction()
               
     self.maps[0].setRange(minLon, maxLon)
     self.maps[1].setRange(minLat, maxLat)  
                
     ir=HydraImageRendererJ3D()
     ir.setMissingColor(255,255,255) # set missing to white
     self.imageRef=subs.addData('a',img,self.airsLevel2Disp,constantMaps=[ConstantMap(-0.2, displayAltType)], renderer=ir)
     subs.addData('b',self.map,self.airsLevel2Disp,constantMaps=makeColorMap(hydra_properties.foreground_color))
     
     colorBar = ColorBarWidget(self.maps[2],hydra_properties.background_color, hydra_properties.foreground_color)
     colorBar.setPreferredSize(Dimension(400,40))
     self.updateColorBar(colorBar)

     hydra_globals.dtuple_cnt += 1
     self.img=img
     self.displayAltType=displayAltType

     if self.refMarkerRef == None:
       rt = geoPointRef_s[1].getData()
       lat = self.displayData.obsMin[0]
       lon = self.displayData.obsMin[1]
       if rt != None:
         vals = rt.getValues()
         lat = vals[1]
         lon = vals[0]
       refMark = RealTuple(RealTupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, RealType.Altitude]), [Real(self.obsLatitudeInstrument, lat), Real(self.obsLongitudeInstrument, lon), Real(RealType.Altitude, 0.25)], None)
       self.refMarkerRef = subs.addData('c',refMark,self.airsLevel2Disp,subs.makeColorMap('magenta')+[ConstantMap(0.25, self.displayAltType)],renderer=DirectManipulationRendererJ3D())
       self.airsLevel2Disp.toggle(self.refMarkerRef, false)
       self.refProfToggleButton.setSelected(0)
     else:
       self.airsLevel2Disp.addReferences(DirectManipulationRendererJ3D(), self.refMarkerRef, subs.makeColorMap('magenta')+[ConstantMap(0.25, self.displayAltType)])
       self.airsLevel2Disp.toggle(self.refMarkerRef, false)
       self.refProfToggleButton.setSelected(0)

     txtCntrl = self.maps[4].getControl()
     txtCntrl.setJustification(TextControl.Justification.LEFT)
     txtCntrl.setAutoSize(1)
     txtCntrl.setFont(Font("sansserif", Font.PLAIN, 14))

     from java.awt import Color
     if self.r_probe != None:
       del self.r_probe
     roamText = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, Float.NaN), Real(self.obsLongitudeInstrument, Float.NaN), Text(self.text, '   %-6.2f' % Float.NaN)])
     self.roamTextRef = subs.addData('a', roamText, self.airsLevel2Disp, subs.makeColorMap(Color(1.0,0.0,1.0)),zlayer = 0.1)
     self.r_probe = roamProbe(self.airsLevel2Disp, self.displayData, self.toolbar, self.roamTextRef, self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text,self.displayData.getImageValue)

     self.shapes = subs.Shapes(self.airsLevel2Disp,self.maps[3])
     self.shapeMarkerIndex = self.shapes.addShape("cross",scale=.03,color="cyan", autoScale=0)
     self.shapeRef = self.shapes.shapeRef[self.shapeMarkerIndex]
     ll = self.displayData.cs.toReference([[self.currentLine],[self.currentElem]])
     lon = ll[1][0]
     lat = ll[0][0]
     rt = geoPointRef_s[0].getData()
     if rt != None:
       vals = rt.getValues()
       lon = vals[0]
       lat = vals[1]
     if self.ds_cs != None:
       ll = self.ds_cs.toReference([ [lon], [lat], [0]])
       x = ll[0][0]
       y = ll[1][0]
     self.shapes.moveShape(self.shapeMarkerIndex,(x,y))

     self.colorMenu.changeColorScalarMap(self.rgbMap)

     self.airsLevel2Disp.enableAction()
     
     self.frame.pack()
     self.frame.show()
     if self.displayData.isProfile: 
       self.frame.setSize(500,700)
     else:
       self.frame.setSize(600,500)
       
     subs.setPointSize(self.airsLevel2Disp, 4)

     self.refToImage = self.imageRef
     transect.update(self.imageRef)

          
  def setColorScale(self,event):
    dragHistLocation(self.maps[2], self.img, imgRef=self.imageRef)
  
  def captureDisplay(self,event):
    Helper.saveDisplay(self.airsLevel2Disp, JFrame().getContentPane())
  
  def changeVariable(self,event):
    self.displayData.paramName=event.source.text
    self.displayData.isProfile=self.displayData.determineIfProfile()
    event.source.setSelected(1)
    if self.displayData.isProfile:
      self.displayData.levelValue=self.displayData.initLevelIndex
      self.levelMenu.setEnabled(1)
    else:
      self.displayData.levelValue=None
      self.levelMenu.setEnabled(0)
    
    # reinitialize observations and display
    self.displayData.observationField=None    
    self.rebuildAIRSLevel2Display()
    ll = self.displayData.cs.toReference([[self.currentLine],[self.currentElem]])
    lat = ll[0][0]
    lon = ll[1][0]
    val = self.displayData.getImageValue(lat,lon).getValue()          
    roamText = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, lat), Real(self.obsLongitudeInstrument, lon), Text(self.text, '   %-6.2f' % val)])
    self.roamTextRef.setData(roamText)    
    ##self.updateImageText()
  
  def changeLevel(self,event):
    # changes level from the menu
    self.displayData.levelValue=self.displayData.pressureLevels.index(float(event.source.text))
    event.source.setSelected(1)
    self.changeLevelImage(0,Math.log(float(event.source.text))/Math.log(10))

  def changeLevelImage(self,dummyValue,observationLevel):
    # changes level if you click on the profile image
    # dummyValue argument needs to be in there due to definition of HandlePickEvent
    observationLevel = 10**observationLevel
    self.selLevel = observationLevel
    self.levelIndx = self.displayData.observationDomain.valueToIndex([[observationLevel],])[0]
    if (self.lastLevelIndx != self.levelIndx) and self.levelIndx >= 0:
      # Rebuild the line display
      self.profileDisp.removeReference(self.lineRef)
      
      self.lastLevelIndx = self.levelIndx
      rangeValues = self.profileMap[0].getRange() # get the range of the plotted values
      rangeValues[0] = self.x0
      rangeValues[1] = self.x1
      xScalar = self.profileMap[0].getScalar()
      y = self.displayData.observationDomain.indexToValue([self.levelIndx])
      pressVal = y[0][0]
      pressVal = Math.log(pressVal)/Math.log(10)
      self.lineRef = subs.drawLine(self.profileDisp,[(0,rangeValues[1]),(pressVal,pressVal)],color="green")
      
      # change the image here
      self.maps[2].resetAutoScale()
      self.airsLevel2Disp.reAutoScale()
      self.frame.setCursor(3)
      syncher = ScalarMapSyncher(self.maps[2])
      self.displayData.getSameImageSegment(self.levelIndx)
      self.frame.setCursor(0)

      syncher.waitForEvent()

      ll = self.displayData.cs.toReference([[self.currentLine],[self.currentElem]])
      lat = ll[0][0]
      lon = ll[1][0]
      val = self.displayData.getImageValue(lat,lon).getValue()          
      roamText = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, lat), Real(self.obsLongitudeInstrument, lon), Text(self.text, '   %-6.2f' % val)])
      self.roamTextRef.setData(roamText)
       
  def imageToSubset(self, line, elem):
    imageDomain = self.displayData.imageField.getDomainSet()
    lens = imageDomain.getLengths()
    dim  = len(lens)
    idx  = imageDomain.valueToIndex([[line], [elem]])[0]
    grid = []
    k = idx;
    for j in xrange(dim-1):
      grid.append( k % lens[j]);
      k = k / lens[j];
    grid.append(k)
                                                                                                                                
    numLine = int(grid[0])
    numElem = int(grid[1])
    return [numLine, numElem]

  def initProfileDiff(self):
    self.profDiffType = RealType.getRealType("diff")
    if self.diffMap != None:
      self.profileDisp.removeMap(self.diffMap)
    self.diffMap = ScalarMap(self.profDiffType, Display.XAxis)
    self.profileDisp.addMap(self.diffMap)
    valid_range = self.displayData.getObsValidRange()
    self.diffMap.setRange(0, valid_range[1])
    self.profileDiffRef = DataReferenceImpl('d')
    self.profileDisp.addReference(self.profileDiffRef, subs.makeColorMap('blue'))
    self.profileDisp.toggle(self.profileDiffRef, false)

  def updateProfileDiff(self):
    d   = self.profileRef.getData() - self.refProfileRef.getData()
    mt  = d.getType()
    nmt = FunctionType(mt.getDomain(), self.profDiffType)
    nd  = d.changeMathType(nmt)
    self.profileDiffRef.setData(nd)

  def profileDiffToggle(self, event):
    if self.profDiffToggleButton.isSelected() == true:
      self.profileDisp.toggle(self.profileDiffRef, true)
    else:
      self.profileDisp.toggle(self.profileDiffRef, false)
  
  def refProfileToggle(self, event):
    if self.refProfToggleButton.isSelected() == true:
      self.profileDisp.toggle(self.refProfileRef, true)
      self.airsLevel2Disp.toggle(self.refMarkerRef, true)
      from DisplayViewSelect import Pick
      self.h_action = Pick(self.toolbar.listeners, actionCommand="pick2", func="grab", icon="./ui/icons/cursor12.jpg", border=CompoundBorder(LineBorder(Color.magenta,2), EmptyBorder(4,4,4,4)))
      self.toolbar.addHAction(self.h_action)
    else:
      self.profileDisp.toggle(self.refProfileRef, false)
      self.airsLevel2Disp.toggle(self.refMarkerRef, false)
      self.toolbar.removeHAction(self.h_action)
      self.h_action = None

  def updateGeoPoint(self, lon, lat, idx):
    geoPointRef_s[idx].setData(RealTuple(RealTupleType.SpatialEarth2DTuple, [lon, lat]))
  
  def updateProfile(self, lonlatTuple):
    if lonlatTuple != None:
      comps = lonlatTuple.getComponents()
      x = comps[0].getValue()
      y = comps[1].getValue()
      if self.ds_cs != None:
        ll = self.ds_cs.toReference([[x],[y],[0]])
        self.changeProfile(ll[0][0], ll[1][0])
      else:
        self.changeProfile(x,y)
      self.updateProfileDiff()

  def updateRefProfile(self, lonlatTuple):
    comps = lonlatTuple.getComponents()
    lon = comps[0].getValue()
    lat = comps[1].getValue()
    xy = self.displayData.cs.fromReference([[lat,],[lon,]])
    numLine = xy[0][0]
    numElem = xy[1][0]
                                                                                                                                
    imageDomain = self.displayData.imageField.getDomainSet()
    lens = imageDomain.getLengths()
    dim  = len(lens)
    idx  = imageDomain.valueToIndex([[numLine], [numElem]])[0]
    grid = []
                                                                                                                                
    k = idx;
    for j in xrange(dim-1):
      grid.append( k % lens[j]);
      k = k / lens[j];
      grid.append(k)
    numLine = int(grid[0])
    numElem = int(grid[1])
    self.refProfDispListnr.currentLine = numLine
    self.refProfDispListnr.currentElem = numElem
    self.refLine = numLine
    self.refElem = numElem
    if self.displayData.isProfile != false:
      self.refProfileRef.setData(self.displayData.getObservations(numLine,numElem,true))
    self.updateProfileDiff()
    self.refMarkerRef.setData(RealTuple(RealTupleType(self.obsLatitudeInstrument, self.obsLongitudeInstrument, RealType.Altitude), [lat, lon, 0.25]))


  def doChangeProfile(self,lon,lat):
    self.shapes.moveShape(self.shapeMarkerIndex,(lon,lat))
    if self.ds_cs != None:
      ll = self.ds_cs.fromReference([[lon], [lat], [0]])
      lon = ll[0][0]
      lat = ll[1][0]
    self.updateGeoPoint(lon, lat, 0)

  def changeProfile(self,lon,lat):
    self.shapes.moveShape(self.shapeMarkerIndex,(lon,lat))
    
    if self.ds_cs != None:
      ll = self.ds_cs.fromReference([[lon], [lat], [0]])
      lon = ll[0][0]
      lat = ll[1][0]

    if self.displayData.cs != None:
      xy = self.displayData.cs.fromReference([[lat,],[lon,]])
      numLine = xy[0][0]
      numElem = xy[1][0]

      imageDomain = self.displayData.imageField.getDomainSet()
      lens = imageDomain.getLengths()
      dim = len(lens)
      idx = imageDomain.valueToIndex([[numLine], [numElem]])[0]
      grid = []

      k = idx;
      for j in xrange(dim-1):
        grid.append( k % lens[j]);
        k = k / lens[j];
      grid.append(k)

      try:
        self.currentLine = int(numLine)
        self.currentElem = int(numElem)
      except: #- off image
        return

      numLine = int(grid[0])
      numElem = int(grid[1])
        
      if numLine >= 0 and numLine < self.dimInstrument[0] and numElem >= 0 and numElem < self.dimInstrument[1]:
        self.frame.setCursor(3)
        if self.displayData.isProfile != false:
          obs=self.displayData.getObservations(numLine,numElem)
             
        self.frame.setCursor(0)
        
        val = self.displayData.imageField.evaluate(RealTuple(self.displayData.imageField.getType().getDomain().getCoordinateSystem().getReference(), [lat, lon]), Data.NEAREST_NEIGHBOR, Data.NO_ERRORS)             
        strg='Lat = %-6.2f  Lon = %-7.2f     Val = %-6.2f  ' % (lat, lon, val)
        strg = strg+self.displayData.imageRangeUnit.toString()
        tt = self.displayData.getTimeObs(numLine,numElem)
        strgTime = ""
        if tt != None:
          strgTime='Time = %-6.2f [offset (s)]' % (tt)
        strgFOV = ""
        ff = self.displayData.getFOVangleObs(numLine,numElem)
        if ff != None:
          strgFOV='FOV angle = %-6.2f [degrees]' % (ff)
      else:
        strg='Lat = %-6.2f  Lon = %-7.2f not in swath' % (lat, lon)
        strgTime='Time not available'
        strgFOV='FOV angle not defined'
 
      val = self.displayData.getImageValue(lat, lon).getValue()
      roamText = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, lat), Real(self.obsLongitudeInstrument, lon), Text(self.text, '   %-6.2f' % val)])
      self.roamTextRef.setData(roamText)
      
  def addImageText(self):
    data = self.displayData
    disp  = self.airsLevel2Disp
    #-- min/max label
    minTuple = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, data.obsMin[0]), Real(self.obsLongitudeInstrument, data.obsMin[1]), Text(self.text, '   %-6.2f' % data.obsMin[2])])
    minMark = RealTuple(RealTupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument]), [Real(self.obsLatitudeInstrument, data.obsMin[0]), Real(self.obsLongitudeInstrument, data.obsMin[1])], None)
    self.minRef=subs.addData('a',minTuple,disp,subs.makeColorMap(Color(0.8, 0.8, 0.0)),zlayer = 0.1)
    self.minMarkRef=subs.addData('a',minMark,disp,subs.makeColorMap(Color(0.8, 0.8, 0.0)),zlayer = 0.1)
    maxMark = RealTuple(RealTupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument]), [Real(self.obsLatitudeInstrument, data.obsMax[0]), Real(self.obsLongitudeInstrument, data.obsMax[1])], None)
    maxTuple = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, data.obsMax[0]), Real(self.obsLongitudeInstrument, data.obsMax[1]), Text(self.text, '   %-6.2f' % data.obsMax[2])])
    self.maxRef=subs.addData('a',maxTuple,disp,subs.makeColorMap(Color(0.2, 0.8, 0.2)),zlayer = 0.1)
    self.maxMarkRef=subs.addData('a',maxMark,disp,subs.makeColorMap(Color(0.2, 0.8, 0.2)),zlayer = 0.1)
  
  def updateImageText(self):
    minTuple = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, self.displayData.obsMin[0]), Real(self.obsLongitudeInstrument, self.displayData.obsMin[1]), Text(self.text, '   %-6.2f' % self.displayData.obsMin[2])])
    minMark = RealTuple(RealTupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument]), [Real(self.obsLatitudeInstrument, self.displayData.obsMin[0]), Real(self.obsLongitudeInstrument, self.displayData.obsMin[1])], None)
    self.minRef.setData(minTuple)
    self.minMarkRef.setData(minMark)
    maxMark = RealTuple(RealTupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument]), [Real(self.obsLatitudeInstrument, self.displayData.obsMax[0]), Real(self.obsLongitudeInstrument, self.displayData.obsMax[1])], None)
    maxTuple = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, self.displayData.obsMax[0]), Real(self.obsLongitudeInstrument, self.displayData.obsMax[1]), Text(self.text, '   %-6.2f' % self.displayData.obsMax[2])])
    self.maxRef.setData(maxTuple)
    self.maxMarkRef.setData(maxMark)

    tup = self.roamTextRef.getData()
    tup_type = tup.getType()
    c0 = tup.getComponent(0)
    c1 = tup.getComponent(1)
    val = self.displayData.getImageValue(c0.getValue(),c1.getValue()).getValue()
    new_tup = Tuple(tup_type, [c0, c1, Text(self.text, '   %-6.2f' % val)])
    self.roamTextRef.setData(new_tup)  

  def doTransect(self, event):
    if self.toolbar != None:
      self.toolbar.select(ActionEvent(self,ActionEvent.ACTION_PERFORMED,"pick"))
      self.toolbar.buttonGroup.setSelected(self.toolbar.click.getModel(), true)
    transect.makeTransectDisplay(self.displayData, self.img, self.imageRef, self.airsLevel2Disp, self.roamTextRef, self, dspAltLev=0.0, altitudeMap=self.altitudeMap)

  def finishTransect(self):
    if self.toolbar != None:
      self.toolbar.select(ActionEvent(self,ActionEvent.ACTION_PERFORMED,"pick"))
      self.toolbar.buttonGroup.setSelected(self.toolbar.click.getModel(), true)

  def getNavigatedImage(self, replace=false):
    return self.img

  def toggleColorBar(self,event):
    if event.getActionCommand() == "On":
      self.framePane.remove(self.toolbar)
      self.southPanel.add(self.colorBar)
      self.southPanel.add(self.toolbar)
      self.framePane.add("South", self.southPanel)
      self.framePane.validate()
      self.colorBarOn = true
    else:
      self.southPanel.remove(self.colorBar)
      self.framePane.remove(self.southPanel)
      self.framePane.add("South", self.toolbar)
      self.framePane.validate()
      self.colorBarOn = false

  def updateColorBar(self, colorBar):
    if self.colorBarOn == true:
      self.southPanel.remove(self.colorBar)
      self.southPanel.remove(self.toolbar)
      self.southPanel.add(colorBar)
      self.southPanel.add(self.toolbar)
      self.southPanel.validate()
    self.colorBar = colorBar

  
  def windowClose(self,event):
    self.frame.dispose()
    runtime.gc()
