import sys

from javax.swing import JPopupMenu, JFrame, JMenuBar, JMenu, JMenuItem, JTextField, JFileChooser, JLabel, JPanel, JSplitPane, JButton
from javax.swing.border import LineBorder
from java.awt import GridLayout, BorderLayout, FlowLayout, Color, Dimension, Canvas, Toolkit, Font
from java.awt.event import InputEvent
from java.awt.datatransfer import DataFlavor
from java.lang import System, Float, Runtime

from visad import *
from visad.bom import HydraImageRendererJ3D
import mapProjections
import subs
import graph
from subs import *
from visad.python.JPythonMethods import *
from ui.colorbar import ColorBarWidget, ColorBar

import MultiSpectralDataSource
import jarray
from hframe import HFrame
from subsetImage import SubsetImage, Subset
from histostretch import linkHistLocation, dragHistLocation

from util import Helper, ScalarMapSyncher
from util import LambertEA, SatProjection, MapProjectionAdapter
from html_hydra import HtmlBrowserWindow

import hydra_properties
hydra_properties.setProperties()

false = 0
true  = 1

global adaptProj
import hydra_globals

runtime=Runtime.getRuntime()
subsetRef=[]
subsetRef.append(None)

class mod07Source:
  def __init__(self,displayData, subset=None):
    self.displayData = displayData
    self.currentProj = "Lambert Equal Area"
    self.currentLine = 0
    self.currentElem = 0
    self.frame  = HFrame("Hydra MODIS Cloud Products",windowClosing=self.windowClose)
    menuBar   = JMenuBar()
    self.frame.setJMenuBar(menuBar)
    self.probe = None

    varMenu = JMenu("Variables")
     
    varsToAdd=displayData.imageRangeTypeDict.keys()
    for i in xrange(displayData.numObsDataSets):
         varMenuItem=JMenuItem(varsToAdd[i],actionPerformed=self.changeVariable)
         varMenuItem.setEnabled(1)
         varMenu.add(varMenuItem)
             
    menuBar.add(varMenu)
    settingsMenu = JMenu("Settings")
    global colorRangeItem
    colorRangeItem = JMenuItem("Set Color Range", actionPerformed=self.setColorScale)
    colorRangeItem.setEnabled(1)
    settingsMenu.add(colorRangeItem)
    menuBar.add(settingsMenu)

    toolMenu = JMenu("Tools")
    toolItem = JMenuItem("Capture Display", actionPerformed=self.captureDisplay)
    toolItem.setEnabled(1)
    toolMenu.add(toolItem)
    menuBar.add(toolMenu)

    # make dummy maps and displays
    self.mod07Map=makeMaps(RealType.Longitude, "x", RealType.Latitude, "y")
    self.mod07Disp=makeDisplay(self.mod07Map)
    
    mod07DspRenderer = self.mod07Disp.getDisplayRenderer()
    mod07DspRenderer.setBackgroundColor(hydra_properties.background_color)
    mod07DspRenderer.setForegroundColor(hydra_properties.foreground_color)
    self.splitPane=JSplitPane(JSplitPane.HORIZONTAL_SPLIT)
    framePane = self.frame.getContentPane()
    framePane.setLayout(BorderLayout())
    #framePane.add(disp.getComponent(),BorderLayout.WEST)
    self.splitPane.add(self.mod07Disp.getComponent())
    self.splitPane.setContinuousLayout(1)
    self.splitPane.setOneTouchExpandable(1)
    self.splitPane.setDividerSize(12)
    framePane.add("Center",self.splitPane)
    
    from DisplayViewSelect import DisplayViewSelectToolBar, RubberBandSubset
    toolbar = DisplayViewSelectToolBar([self.mod07Disp])
    framePane.add("North", toolbar)

    self.toolbar=toolbar
    self.text = TextType.Generic
    self.map=load('./data/outlsupw')
    
    self.rebuildMod07Display()
    self.mapRef=subs.addData('m',self.map,self.mod07Disp,constantMaps=makeColorMap(hydra_properties.foreground_color)+[ConstantMap(-0.1, self.displayAltType)])

    #finally, we want to have a small 'cross' to show the position
    self.shapes = subs.Shapes(self.mod07Disp,self.maps[3])
    self.shapeMarkerIndex = self.shapes.addShape("cross",scale=.03,color="cyan", autoScale=0)
    self.shapeRef = self.shapes.shapeRef[self.shapeMarkerIndex]
    ll = self.displayData.cs.toReference([[self.currentLine],[self.currentElem]])
    if self.ds_cs != None:
      ll = self.ds_cs.toReference([ [ll[1][0] ], [ ll[0][0] ], [0]])
    self.shapes.moveShape(self.shapeMarkerIndex,(ll[0][0],ll[1][0]))
 
    self.addImageText() 

    """
    roamText = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, Float.NaN), 
                Real(self.obsLongitudeInstrument, Float.NaN), Text(self.text, '   %-6.2f' % Float.NaN)])
    self.roamTextRef = subs.addData('a', roamText, self.mod07Disp, subs.makeColorMap(Color(1.0,0.0,1.0)),zlayer = 0.1)

    class roamProbe(DisplayListener):
      def __init__(self, display, ad, toolbar, textRef, y, x, txt):
        self.display = display
        self.display.addDisplayListener(self)
        self.dr = display.getDisplayRenderer()
        self.ad = ad
        self.toolbar = toolbar
        self.textRef = textRef
        self.x = x
        self.y = y
        self.txt = txt
        self.last_val = None

      def displayChanged(self, event):
        if event.getId() == 2 or event.getId() == 3:
          return
        lat = self.dr.getDirectAxisValue("Latitude")
        lon = self.dr.getDirectAxisValue("Longitude")
        if self.toolbar.pickAction.enabled == 1:
          val = self.ad.getImageValue(lat,lon).getValue()
          if val != self.last_val:
            self.textRef.setData(Tuple(TupleType([self.y, self.x, self.txt]), [Real(self.y, lat), Real(self.x, lon), Text(self.txt, '   %-6.2f' % val)]))
            self.last_val = val

    roamProbe(self.mod07Disp, self.displayData, self.toolbar, self.roamTextRef, self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text)
    """
   
    subs.HandlePickEvent(self.mod07Disp, self.doPickEvent)

  def rebuildMod07Display(self):
    # clear display before creating a new one
    self.mod07Disp.removeAllReferences()
    self.mod07Disp.clearMaps()
    
    testForColorbar=self.splitPane.getRightComponent()
    if testForColorbar != None:
        self.splitPane.remove(testForColorbar)
    #obsElementInstrument, obsLineInstrument, obsChannelIndexInstrument, observationChannelsInstrument, observationsInstrument = displayData.getRealTypes()
    self.obsLatitudeInstrument, self.obsLongitudeInstrument  = self.displayData.getGeoRealTypes()
 
    if self.displayData.numObsElements < 512:
       img = self.displayData.getLevel2ImageSegment(0)
    else:
       img = self.displayData.getLevel2ImageSegment(0,Subset(None,None,None,None,step=10))

    self.observationsInstrument = self.displayData.imageRangeType

    corners = mapProjections.getValidCorners(self.displayData.gs, [[-180,180], [-90, 90]])
     
    minLon = min(corners[0])
    minLat = min(corners[1])
    maxLon = max(corners[0])
    maxLat = max(corners[1])

    displayLatType   = DisplayRealType("ProjectionLat_%d" % hydra_globals.dtuple_cnt, true, minLat, maxLat, 0.0, CommonUnit.degree)
    displayLonType   = DisplayRealType("ProjectionLon_%d" % hydra_globals.dtuple_cnt, true, minLon, maxLon, 0.0, CommonUnit.degree)
    displayAltType   = DisplayRealType("ProjectionAlt_%d" % hydra_globals.dtuple_cnt, true, -1.0, 1.0, 0.0, None)

    adaptProj = MapProjectionAdapter(LambertEA(corners))
     
    self.ds_cs = adaptProj
     
    displayTupleType = DisplayTupleType([displayLonType, displayLatType, displayAltType], adaptProj)

    self.maps = subs.makeMaps(self.obsLongitudeInstrument,'x', self.obsLatitudeInstrument, 'y', self.observationsInstrument, 'rgb','marker','shape', self.text, 'text',RealType.XAxis, 'x', RealType.YAxis, 'y')
     #- replace first two with user-defined display types
    self.maps[0] = ScalarMap(self.obsLongitudeInstrument, displayLonType)
    self.maps[1] = ScalarMap(self.obsLatitudeInstrument, displayLatType)
    self.maps[5].setRange(-1, 1)
    self.maps[6].setRange(-1, 1)
    self.maps = self.maps+[ScalarMap(RealType.Altitude, displayAltType)]
     
    self.mod07Disp.addMaps(self.maps)
     
     # get the dimensions
    dimInstrumentDomain = img.getDomainSet()
    self.dimInstrument = dimInstrumentDomain.getLengths()

    self.mod07Disp.disableAction()

    self.maps[0].setRange(minLon, maxLon)
    self.maps[1].setRange(minLat, maxLat)
     
    ir=HydraImageRendererJ3D()
    ir.setMissingColor(0,0,0) # set missing to black (Liam request)
    imageRef=subs.addData('a',img,self.mod07Disp, constantMaps=[ConstantMap(-0.2, displayAltType)], renderer=ir)
    subs.addData('b',self.map,self.mod07Disp,constantMaps=makeColorMap(hydra_properties.foreground_color))

    self.shapes = subs.Shapes(self.mod07Disp,self.maps[3])
    self.shapeMarkerIndex = self.shapes.addShape("cross",scale=.03,color="cyan", autoScale=0)
    self.shapeRef = self.shapes.shapeRef[self.shapeMarkerIndex]
    ll = self.displayData.cs.toReference([[self.currentLine],[self.currentElem]])
    if self.ds_cs != None:
      ll = self.ds_cs.toReference([ [ll[1][0] ], [ ll[0][0] ], [0]])
    self.shapes.moveShape(self.shapeMarkerIndex,(ll[0][0],ll[1][0]))
                                                                                                                                                        
    txtCntrl = self.maps[4].getControl()
    txtCntrl.setJustification(TextControl.Justification.LEFT)
    txtCntrl.setAutoSize(1)
    txtCntrl.setFont(Font("sansserif", Font.PLAIN, 14))
    txtCntrl.setSize(2.0)

    self.addImageText()
                                                                                                                                                        
    roamText = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, Float.NaN),
                Real(self.obsLongitudeInstrument, Float.NaN), Text(self.text, '   %-6.2f' % Float.NaN)])
    self.roamTextRef = subs.addData('a', roamText, self.mod07Disp, subs.makeColorMap(Color(1.0,0.0,1.0)),zlayer = 0.1)

    if self.probe != None:
      self.mod07Disp.removeDisplayListener(self.probe)

    import roamProbe
    from roamProbe import roamProbe
    self.probe = roamProbe(self.mod07Disp, self.displayData, self.toolbar, self.roamTextRef, self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text)

    colorBar = ColorBarWidget(self.maps[2],hydra_properties.background_color, hydra_properties.foreground_color)
    colorBar.setPreferredSize(Dimension(20,400))
    self.splitPane.add(colorBar)
    
    hydra_globals.dtuple_cnt += 1
    self.img=img
    self.displayAltType=displayAltType
     
    self.frame.pack()
    self.frame.setSize(600,500)
    self.frame.show()
    self.mod07Disp.enableAction()
    self.splitPane.setDividerLocation(250)

  def windowClose(self,event):
    self.frame.dispose()
    runtime.gc()

  def setColorScale(self,event):
    dragHistLocation(self.maps[2], self.img)
  
  def captureDisplay(self,event):
    Helper.saveDisplay(self.mod07Disp, JFrame().getContentPane())

  def changeVariable(self,event):
    self.displayData.paramName=event.source.text
    event.source.setSelected(1)
    
    # reinitialize observations and display
    self.displayData.observationField=None    
    self.rebuildMod07Display()
    ll = self.displayData.cs.toReference([[self.currentLine],[self.currentElem]])
    lat = ll[0][0]
    lon = ll[1][0]
    val = self.displayData.getImageValue(lat,lon).getValue()          
    roamText = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, lat), Real(self.obsLongitudeInstrument, lon), Text(self.text, '   %-6.2f' % val)])
    self.roamTextRef.setData(roamText)    
    self.updateImageText()

  def addImageText(self):
    data = self.displayData
    disp  = self.mod07Disp
    #-- min/max label
    minTuple = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, data.obsMin[0]), Real(self.obsLongitudeInstrument, data.obsMin[1]), Text(self.text, '   %-6.2f' % data.obsMin[2])])
    minMark = RealTuple(RealTupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument]), [Real(self.obsLatitudeInstrument, data.obsMin[0]), Real(self.obsLongitudeInstrument, data.obsMin[1])], None)
    self.minRef=subs.addData('a',minTuple,disp,subs.makeColorMap(Color(0.8, 0.8, 0.0)),zlayer = 0.1)
    self.minMarkRef=subs.addData('a',minMark,disp,subs.makeColorMap(Color(0.8, 0.8, 0.0)),zlayer = 0.1)
    maxMark = RealTuple(RealTupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument]), [Real(self.obsLatitudeInstrument, data.obsMax[0]), Real(self.obsLongitudeInstrument, data.obsMax[1])], None)
    maxTuple = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, data.obsMax[0]), Real(self.obsLongitudeInstrument, data.obsMax[1]), Text(self.text, '   %-6.2f' % data.obsMax[2])])
    self.maxRef=subs.addData('a',maxTuple,disp,subs.makeColorMap(Color(0.2, 0.8, 0.2)),zlayer = 0.1)
    self.maxMarkRef=subs.addData('a',maxMark,disp,subs.makeColorMap(Color(0.2, 0.8, 0.2)),zlayer = 0.1)

  def updateImageText(self):
    minTuple = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, self.displayData.obsMin[0]), Real(self.obsLongitudeInstrument, self.displayData.obsMin[1]), Text(self.text, '   %-6.2f' % self.displayData.obsMin[2])])
    minMark = RealTuple(RealTupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument]), [Real(self.obsLatitudeInstrument, self.displayData.obsMin[0]), Real(self.obsLongitudeInstrument, self.displayData.obsMin[1])], None)
    self.minRef.setData(minTuple)
    self.minMarkRef.setData(minMark)
    maxMark = RealTuple(RealTupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument]), [Real(self.obsLatitudeInstrument, self.displayData.obsMax[0]), Real(self.obsLongitudeInstrument, self.displayData.obsMax[1])], None)
    maxTuple = Tuple(TupleType([self.obsLatitudeInstrument, self.obsLongitudeInstrument, self.text]), [Real(self.obsLatitudeInstrument, self.displayData.obsMax[0]), Real(self.obsLongitudeInstrument, self.displayData.obsMax[1]), Text(self.text, '   %-6.2f' % self.displayData.obsMax[2])])
    self.maxRef.setData(maxTuple)
    self.maxMarkRef.setData(maxMark)

    tup = self.roamTextRef.getData()
    tup_type = tup.getType()
    c0 = tup.getComponent(0)
    c1 = tup.getComponent(1)
    val = self.displayData.getImageValue(c0.getValue(),c1.getValue()).getValue()
    new_tup = Tuple(tup_type, [c0, c1, Text(self.text, '   %-6.2f' % val)])
    self.roamTextRef.setData(new_tup)  

  def doPickEvent(self,lon,lat):
    self.shapes.moveShape(self.shapeMarkerIndex,(lon,lat))

  def changeSubset(a,b):
    pass
