from java.lang import Double
from visad import DisplayListener, DisplayEvent, CellImpl, Real, ConstantMap, Display, Gridded1DSet, FlatField, RealTupleType, FunctionType
from visad.java3d import DirectManipulationRendererJ3D
from visad.java2d import DirectManipulationRendererJ2D
import subs
from subs import *
import jarray
from visad.python.JPythonMethods import *
from javax.swing import JFrame, JTextField, JPanel, JMenuBar, JMenu, JMenuItem, JLabel, JButton
from javax.swing import JTabbedPane
from java.awt import BorderLayout, FlowLayout, GridLayout, Color
from javax.swing.border import LineBorder
import graph
from java.lang.Math import abs
from org.python.core import PyTuple, PyList

import hydra_properties
from hydra_properties import true, false
from util import ScalarMapSyncher, Helper


def make3channelHistogram(scalarMap_s, img, updateWithDrag=false, imgRef=None):

  frame = JFrame("3-channel hist-stretch tool")
  tabPane = JTabbedPane()
  labels = ['red', 'green', 'blue']

  for i in xrange(3):
    dH = dragHistLocation2(scalarMap_s[i], img, updateWithDrag, imgRef, rangeTupleIdx=i)
    tabPane.addTab(labels[i], dH.mainPanel)


  tabPane.doLayout()
  min_size = tabPane.getMinimumSize()
  frame.setSize(min_size.width, min_size.height)
  frame.setContentPane(tabPane)
  frame.pack()
  frame.setVisible(1)


class dragHistLocation2(CellImpl, DisplayListener):
  def __init__(self, scalarMap, img, updateWithDrag=false, imgRef=None, rangeTupleIdx=None):
    self.scalarMap      = scalarMap
    self.updateWithDrag = updateWithDrag
    self.setup          = 0
    self.bins           = 20
    self.botPanel       = JPanel()
    self.display        = None
    self.histRef        = None
    self.imgRef         = imgRef
    self.tupIdx         = rangeTupleIdx
    self.mainPanel      = JPanel()
    self.mainPanel.setLayout(BorderLayout())

    self.panel = self.buildDisplay(img, self.bins, bottom=self.botPanel,clip=0,color=hydra_properties.foreground_color,tupIdx=self.tupIdx)

    self.display.addDisplayListener(self)
    dsp_rdr = self.display.getDisplayRenderer()
    dsp_rdr.setBackgroundColor(hydra_properties.background_color)
    dsp_rdr.setForegroundColor(hydra_properties.foreground_color)
    
    lowhi = self.scalarMap.getRange()
    
    self.leftSelectRef = self.display.addData("left", Real(self.xmap.getScalar(), lowhi[0]),
      constantMaps=[ConstantMap(-1, Display.YAxis)]+[ConstantMap(8,Display.PointSize)]+makeColorMap("magenta"), renderer=DirectManipulationRendererJ2D(), zlayer=0.1)
      
    self.rghtSelectRef = self.display.addData("rght", Real(self.xmap.getScalar(), lowhi[1]),
      constantMaps=[ConstantMap(-1, Display.YAxis)]+[ConstantMap(8,Display.PointSize)]+makeColorMap("green"), renderer=DirectManipulationRendererJ2D(), zlayer=0.1)
          
    self.addReference(self.leftSelectRef)
    self.addReference(self.rghtSelectRef)

    #- make left or rght drag
    self.mouseHelper = dsp_rdr.getMouseBehavior().getMouseHelper()
    self.mouseHelper.setFunctionMap([[[MouseHelper.DIRECT, MouseHelper.ZOOM],
                                      [MouseHelper.TRANSLATE, MouseHelper.NONE]],
                                     [[MouseHelper.CURSOR_TRANSLATE, MouseHelper.CURSOR_ZOOM],
                                      [MouseHelper.NONE, MouseHelper.NONE]],
                                     [[MouseHelper.DIRECT, MouseHelper.DIRECT],
                                      [MouseHelper.DIRECT, MouseHelper.DIRECT]]])
    
    manual = 1
    if manual == 1:
      self.leftSelectTextField = JTextField(actionPerformed=self.leftSelect)
      self.leftSelectTextField.setBorder(LineBorder(Color.magenta,2))
      self.leftSelectTextField.setText('%8.3f' % lowhi[0])
      self.rghtSelectTextField = JTextField(actionPerformed=self.rghtSelect)
      self.rghtSelectTextField.setBorder(LineBorder(Color.green,2))
      
      self.rghtSelectTextField.setText('%8.3f' % lowhi[1])
      textPanel = JPanel()
      textPanel.setLayout(FlowLayout())
      textPanel.add(self.leftSelectTextField)
      textPanel.add(self.rghtSelectTextField)
      self.botPanel.add(textPanel)

    self.mainPanel.add(self.panel, BorderLayout.CENTER)
    self.mainPanel.add(textPanel, BorderLayout.SOUTH)
      
    self.lval_last = lowhi[0]
    self.rval_last = lowhi[1]
    self.setup = 1   #- don't trigger the doAction below in the init

    if imgRef != None:
      class cc(CellImpl):
        def __init__(self, parent):
          self.imgRef    = parent.imgRef
          self.scalarMap = parent.scalarMap
          self.parent    = parent
        
        def doAction(self):
          syncher = ScalarMapSyncher(self.parent.scalarMap)
          syncher.waitForEvent()
          self.parent.updateDisplay(self.parent.imgRef.getData())
      self.imgChangeListnr = cc(self)
      self.imgChangeListnr.addReference(imgRef)

  def updateRef(self, imgRef, scalarMap):
    self.scalarMap = scalarMap
    self.imgRef = imgRef

    class cc(CellImpl):
      def __init__(self, parent):
        self.imgRef    = parent.imgRef
        self.scalarMap = parent.scalarMap
        self.parent    = parent

      def doAction(self):
        syncher = ScalarMapSyncher(self.parent.scalarMap)
        syncher.waitForEvent()
        self.parent.updateDisplay(self.parent.imgRef.getData())
    self.imgChangeListnr = cc(self)
    self.imgChangeListnr.addReference(imgRef)

    self.buildDisplay(imgRef.getData(), self.bins, bottom=self.botPanel,clip=0,color=hydra_properties.foreground_color,tupIdx=self.tupIdx)

  def captureDisplay(self, event):
    Helper.saveDisplay(self.display, JFrame().getContentPane())

  def changeXaxisLabel(self,event):
    if event.getActionCommand() == 'default':
      self.xAxisScale.setLabel(self.dfltXaxisLabel)
      self.xAxisTextField.setText(self.dfltXaxisLabel)
    else:
      self.xAxisScale.setLabel(self.xAxisTextField.getText())

  def displaySettings(self, event):
    frame = JFrame("Display Settings")
    pane = frame.getContentPane()
    pane.setLayout(GridLayout(3,1,2,2))
    pane.add(JLabel("x-axis label:"))
    self.xAxisTextField = JTextField(self.xAxisScale.getLabel(), actionPerformed=self.changeXaxisLabel)
    pane.add(self.xAxisTextField)
    setButton = JButton("set",actionPerformed=self.changeXaxisLabel)
    dfltButton = JButton("default",actionPerformed=self.changeXaxisLabel)
    subPanel = JPanel(FlowLayout())
    subPanel.add(setButton)
    subPanel.add(dfltButton)
    pane.add(subPanel)
    pane.doLayout()
    min_size = pane.getMinimumSize()
    frame.setSize(min_size.width+30, min_size.height+5)
    frame.setVisible(1)

    

  def buildDisplay(self, data, bins=20, width=400, height=400, title="VisAD Histogram", color=None, bottom=None, top=None, panel=None, clip=1,tupIdx=None):
    if isinstance(data,PyList) or isinstance(data,PyTuple):
      data = field(data)
    x=[]
    y=[]

    if tupIdx != None:
      h = hist(data, [tupIdx], [bins])
    else:
      h = hist(data, [0], [bins])

    dom = getDomain(h)
    d = dom.getSamples()
    step2 = dom.getStep()/2
    hmin = h[0].getValue()
    hmax = hmin

    for i in range(0,len(h)):
      hval = h[i].getValue()
      if hval < hmin: hmin = hval
      if hval > hmax: hmax = hval

    for i in range(0,len(h)):
      xm = d[0][i]-step2
      xp = d[0][i]+step2
      x.append(xm)
      y.append(hmin)
      x.append(xm)
      hval = h[i].getValue()
      y.append(hval)
      x.append(xp)
      y.append(hval)
      x.append(xp)
      y.append(hmin)

    domt = domainType(h)
    rngt = rangeType(h)

    xmap = ScalarMap(domt[0], Display.XAxis)
    ymap = ScalarMap(rngt, Display.YAxis)
    ymap.setRange(hmin, hmax + abs(hmax * .05))

    if self.display == None:
      disp = subs.makeDisplay2D( (xmap, ymap) )
      self.histRef = subs.drawLine(disp, (x,y), mathtype=(domt[0],rngt), color=color)
      showAxesScales(disp,1)
      subs.setBoxSize(disp,.65,clip)
      subs.setAspectRatio(disp, float(width)/float(height))
      menuBar = JMenuBar()
      toolMenu = JMenu("Tools")
      menuItem = JMenuItem("Capture Display", actionPerformed=self.captureDisplay)
      menuItem.setEnabled(1)
      toolMenu.add(menuItem)
      menuBar.add(toolMenu)
      settingsMenu = JMenu("Settings")
      menuItem = JMenuItem("Display", actionPerformed=self.displaySettings)
      menuItem.setEnabled(1)
      settingsMenu.add(menuItem)
      menuBar.add(settingsMenu)
      top = menuBar
      #-subs.showDisplay(disp,width,height,title,bottom,top,panel)
      self.display = disp
    else:
      self.display.removeAllReferences()
      self.display.removeMap(self.xmap)
      self.display.removeMap(self.ymap)
      as = xmap.getAxisScale()
      as.setVisible(1)
      as = ymap.getAxisScale()
      as.setVisible(1)

      lohi = self.scalarMap.getRange()
      xmap.setRange(lohi[0]-step2, lohi[1]+step2)
      self.display.addMap(xmap)
      self.display.addMap(ymap)
      self.histRef = subs.drawLine(self.display, (x,y), mathtype=(domt[0],rngt), color=color)

      self.leftSelectRef = self.display.addData("left", Real(xmap.getScalar(), lohi[0]),
      constantMaps=[ConstantMap(-1, Display.YAxis)]+[ConstantMap(8,Display.PointSize)]+makeColorMap("magenta"), renderer=DirectManipulationRendererJ2D(), zlayer=0.1)

      self.rghtSelectRef = self.display.addData("rght", Real(xmap.getScalar(), lohi[1]),
      constantMaps=[ConstantMap(-1, Display.YAxis)]+[ConstantMap(8,Display.PointSize)]+makeColorMap("green"), renderer=DirectManipulationRendererJ2D(), zlayer=0.1)

      self.addReference(self.leftSelectRef)
      self.addReference(self.rghtSelectRef)

    self.xmap = xmap
    self.ymap = ymap
    self.xAxisScale = xmap.getAxisScale()
    self.dfltXaxisLabel = self.xAxisScale.getLabel()

    #-return panel
    return self.display.getComponent()
 

  def updateDisplay(self, data):
    x=[]
    y=[]

    h = hist(data, [0], [self.bins])
    dom = getDomain(h)
    d = dom.getSamples()
    step2 = dom.getStep()/2

    hmin = h[0].getValue()
    hmax = hmin

    for i in range(0,len(h)):
      hval = h[i].getValue()
      if hval < hmin: hmin = hval
      if hval > hmax: hmax = hval
                                                                                                                                                   
    for i in range(0,len(h)):
      xm = d[0][i]-step2
      xp = d[0][i]+step2
      x.append(xm)
      y.append(hmin)
      x.append(xm)
      hval = h[i].getValue()
      y.append(hval)
      x.append(xp)
      y.append(hval)
      x.append(xp)
      y.append(hmin)

    self.ymap.setRange(hmin, hmax + abs(hmax * .05))
    self.display.moveLine(self.histRef, (x,y))

    lohi = self.scalarMap.getRange()
    self.xmap.setRange(lohi[0]-step2, lohi[1]+step2)

    type = self.leftSelectRef.getData().getType()
    self.leftSelectRef.setData(Real(type, lohi[0]))

    type = self.rghtSelectRef.getData().getType()
    self.rghtSelectRef.setData(Real(type, lohi[1]))
            
  def doAction(self):
    if self.setup == 1:
      lval = self.leftSelectRef.getData().getValue()
      self.leftSelectTextField.setText('%8.3f' % lval)
      rval = self.rghtSelectRef.getData().getValue() 
      self.rghtSelectTextField.setText('%8.3f' % rval)
      if self.updateWithDrag == true:
        self.update(lval, rval)

  def displayChanged(self, event):
    if self.updateWithDrag == false:
      if event.getId() == DisplayEvent.MOUSE_RELEASED:
        lval = self.leftSelectRef.getData().getValue()
        rval = self.rghtSelectRef.getData().getValue()
        if lval != self.lval_last or rval != self.rval_last:
          self.update(lval, rval)
          self.lval_last = lval
          self.rval_last = rval
      
  def leftSelect(self,event):
    txt = self.leftSelectTextField.getText()
    val = float(txt)
    type = self.leftSelectRef.getData().getType()
    self.leftSelectRef.setData(Real(type, val))
    if self.updateWithDrag == false:
      self.lval_last = val
      self.update(self.lval_last, self.rval_last)
    
  def rghtSelect(self,event):
    txt = self.rghtSelectTextField.getText()
    val = float(txt)
    type = self.rghtSelectRef.getData().getType()
    self.rghtSelectRef.setData(Real(type, val))
    if self.updateWithDrag == false:
      self.rval_last = val
      self.update(self.lval_last, self.rval_last)

  def update(self, lval, rval):
    self.scalarMap.setRange(lval, rval)

      
class dragHistLocationColorFunction(dragHistLocation2):
  def __init__(self, scalarMap, img, vminmaxbig, colorTable, colorscaleselect=None):
    dragHistLocation2.__init__(self, scalarMap, img)
    self.vminmaxbig = vminmaxbig
    self.ct = colorTable
    self.colorscaleselect = colorscaleselect
    
  def update(self, lval, rval):
    cc = self.scalarMap.getControl()
    ct_len = len(self.ct[0])
    ct = self.colorscaleselect.currentTable
                                                                                                                                    
    vmin = self.vminmaxbig[0]
    vmax = self.vminmaxbig[1]
    vdiff = vmax - vmin
    vinc  = vdiff/(ct_len - 1)
                                                                                                                                    
    if rval <= vmax:
      fsmpls = jarray.zeros(ct_len+7,'f')
      fsmpls[0] = -10000.0
      fsmpls[1] = 0.0
      for i in xrange(ct_len-1): fsmpls[i+2] = fsmpls[i+1] + vinc/(vmax - vmin)
      fsmpls[ct_len+1] = (vmax - lval)/(rval - lval)
      fsmpls[ct_len+1] *= 1.0001
      fsmpls[ct_len+2] = (self.vminmaxbig[2] - lval)/(rval - lval)
      fsmpls[ct_len+3] = (self.vminmaxbig[3] - lval)/(rval - lval)
      fsmpls[ct_len+4] = (self.vminmaxbig[4] - lval)/(rval - lval)
      fsmpls[ct_len+5] = (self.vminmaxbig[5] - lval)/(rval - lval)
      fsmpls[ct_len+6] = (self.vminmaxbig[6] - lval)/(rval - lval)
      clr_set = Gridded1DSet(RealType.Generic, [fsmpls], ct_len+7)
      new_ct  = [[], [], []]
                                                                                                                                    
      new_ct[0].append(ct[0][0]), new_ct[1].append(ct[1][0]), new_ct[2].append(ct[2][0])
      for i in range(ct_len): new_ct[0].append(ct[0][i]), new_ct[1].append(ct[1][i]), new_ct[2].append(ct[2][i])
      new_ct[0].append(ct[0][ct_len-1]), new_ct[1].append(ct[1][ct_len-1]), new_ct[2].append(ct[2][ct_len-1])
      new_ct[0].append(0.3), new_ct[1].append(0.0), new_ct[2].append(0.3)
      new_ct[0].append(0.5), new_ct[1].append(0.5), new_ct[2].append(0.0)
      new_ct[0].append(0.0), new_ct[1].append(0.5), new_ct[2].append(0.5)
      new_ct[0].append(1.0), new_ct[1].append(0.0), new_ct[2].append(0.0)
      new_ct[0].append(0.0), new_ct[1].append(0.0), new_ct[2].append(1.0)
    else:
      fsmpls = jarray.zeros(ct_len+6,'f')
      fsmpls[0] = -10000.0
      fsmpls[1] = 0.0
      for i in xrange(ct_len-1): fsmpls[i+2] = fsmpls[i+1] + vinc/(vmax - vmin)
      fsmpls[ct_len+1] = (self.vminmaxbig[2] - lval)/(rval - lval)
      fsmpls[ct_len+2] = (self.vminmaxbig[3] - lval)/(rval - lval)
      fsmpls[ct_len+3] = (self.vminmaxbig[4] - lval)/(rval - lval)
      fsmpls[ct_len+4] = (self.vminmaxbig[5] - lval)/(rval - lval)
      fsmpls[ct_len+5] = (self.vminmaxbig[6] - lval)/(rval - lval)
      clr_set = Gridded1DSet(RealType.Generic, [fsmpls], ct_len+6)
      new_ct  = [[], [], []]
                                                                                                                                    
      new_ct[0].append(ct[0][0]), new_ct[1].append(ct[1][0]), new_ct[2].append(ct[2][0])
      for i in range(ct_len): new_ct[0].append(ct[0][i]), new_ct[1].append(ct[1][i]), new_ct[2].append(ct[2][i])
      new_ct[0].append(0.3), new_ct[1].append(0.0), new_ct[2].append(0.3)
      new_ct[0].append(0.5), new_ct[1].append(0.5), new_ct[2].append(0.0)
      new_ct[0].append(0.0), new_ct[1].append(0.5), new_ct[2].append(0.5)
      new_ct[0].append(1.0), new_ct[1].append(0.0), new_ct[2].append(0.0)
      new_ct[0].append(0.0), new_ct[1].append(0.0), new_ct[2].append(1.0)
                                                                                                                                    
    clr_fnc = FlatField(FunctionType(RealType.Generic, RealTupleType([makeType("cR"), makeType("cG"), makeType("cB")])), clr_set)
    clr_fnc.setSamples(new_ct)
                                                                                                                                    
    self.display.disableAction()
    cc.setFunction(clr_fnc)
    self.scalarMap.setRange(lval, rval)
    self.display.enableAction()
