from java.awt import Color, Font
from visad import ControlListener, DisplayListener, DataReferenceImpl, Gridded2DSet, RealTupleType, UnionSet
from visad import RealType, TextType, TupleType, Real, Text, Tuple, DisplayEvent, ScalarMap, Display, TextControl
from visad.java3d import MouseBehaviorJ3D
from visad.bom import ScreenLockedRendererJ3D
from java.lang import Math
import subs
import jarray
import java
                                                                                                                                      
from hydra_properties import true, false
from hydra_globals import color_scale_dict


class ImageLabel:
  def __init__(self, display, dsp_cs, init_scale, instrName, date_str, time_str, lonType, latType, colorScale):
    self.d = display
    self.init_scale = init_scale
    self.ds_cs   = dsp_cs
    self.enabled = false
    self.proj_cntrl = self.d.getProjectionControl()
    self.rot = jarray.zeros(3, 'd')
    self.scl = jarray.zeros(1, 'd')
    self.trn = jarray.zeros(3, 'd')
    self.dLabelText = TextType.getTextType("imgLabel")
    self.imgLblRef = None
    self.obsLongitudeInstrument = lonType
    self.obsLatitudeInstrument = latType
    self.colorScale = colorScale

    if ((instrName == None) or (date_str == None) or (time_str == None)):
      self.enableable = false
      self.imgLbl = None
      return
    else:
      self.imgLbl = instrName+'  '+date_str+'  '+time_str
      self.enableable = true

    dr = self.d.getDisplayRenderer()
    self.cnvs = dr.getCanvas()
    self.mBhvr = dr.getMouseBehavior()
    if self.init_scale == None:
      mtr = self.proj_cntrl.getMatrix()
      MouseBehaviorJ3D.unmake_matrix(self.rot, self.scl, self.trn, mtr)
      self.init_scale = self.scl[0]


  def __del__(self):
    self.disable()

  def enable(self, dsp_cs=None):
    if self.enableable == false:
      return

    self.enabled = true
    mtr = self.proj_cntrl.getMatrix()
    MouseBehaviorJ3D.unmake_matrix(self.rot, self.scl, self.trn, mtr)
    scale = self.scl[0]
    if dsp_cs != None:
      self.ds_cs = dsp_cs

    self.imgLblTxtMap = ScalarMap(self.dLabelText, Display.Text)
    self.d.addMap(self.imgLblTxtMap)
    tcntrl = self.imgLblTxtMap.getControl()
    tcntrl.setJustification(TextControl.Justification.CENTER)
    tcntrl.setFont(Font("sansserif", Font.PLAIN, 14))
    ##tcntrl.setSize(2.0*(self.init_scale/scale))
    tcntrl.setSize(2.0)
    self.tcntrl = tcntrl
    self.update()

  def disable(self):
    if self.enabled == false:
      return
    self.enabled = false
    self.d.removeReference(self.imgLblRef)
    self.d.removeMap(self.imgLblTxtMap)

  def update(self, dsp_cs=None):
    if self.enabled == false:
      return

    size = self.cnvs.getSize()
    vray = self.mBhvr.findRay(size.width/2, 15)
    x = vray.position[0]
    y = vray.position[1]
    lola = self.ds_cs.fromReference([[x],[y],[0]])

    lon = lola[0][0]
    lat = lola[1][0]

    imgLabelTxt = Tuple(TupleType([self.obsLongitudeInstrument, self.obsLatitudeInstrument, self.dLabelText]), [Real(self.obsLongitudeInstrument, lon), Real(self.obsLatitudeInstrument, lat), Text(self.dLabelText, self.imgLbl)])

    mtr = self.proj_cntrl.getMatrix()
    MouseBehaviorJ3D.unmake_matrix(self.rot, self.scl, self.trn, mtr)
    scale = self.scl[0]

    color = (color_scale_dict[self.colorScale.current])['img_label_color']

    ##self.tcntrl.setSize(2.0*self.init_scale/scale)
    self.tcntrl.setSize(2.0)
    if self.imgLblRef != None:
      self.d.removeReference(self.imgLblRef)
    self.imgLblRef = subs.addData('lbl', imgLabelTxt, self.d, subs.makeColorMap(color),zlayer=0.1, renderer=ScreenLockedRendererJ3D())
