import jarray
import struct
from util import Helper
from visad import *
from visad.util import DataUtility
from java.io import FileInputStream
from java.util import Properties
from visad.python.JPythonMethods import *
from java.lang import Float, Math, System, Object
from java.net import URL
from visad.data.units import *

from subsetImage import Subset
import hydra_globals
from hydra_globals import hydra_path
                                                                                                                   
false = 0
true  = 1


from MultiSpectralDataSource import ModisData
class ModisData500m(ModisData):
  def __init__(self, filename, reader, readerTable,master=None):
    ModisData.__init__(self, filename, reader, readerTable, master)

    self.geoOffset = 0

  def getObservationChannels(self):
    channels = []
    self.band_numbers = []
    ff       = open(hydra_path+'/ancillary/modis/default_cntr_wn_bw_500m.txt', 'r')
    lines    = ff.readlines()
    nlines   = len(lines)
    self.bandDict = {}
    cnt = 0
    for i in xrange(nlines):
      if lines[i].startswith('!'):
        pass
      else:
        fields      = lines[i].split()
        band_number = fields[0]
        center_wn   = fields[1]
        low_wn      = fields[2]
        hi_wn       = fields[3]
        center_wn   = float(center_wn)
        self.band_numbers.append(band_number)
        channels.append(center_wn)
        self.bandDict.update({band_number : [cnt,center_wn]})
        self.bandDict.update({center_wn : band_number})
        cnt += 1
                                                                                                                   
    self.fileIndexes = QuickSort.sort(channels)
    new_channels     = jarray.zeros(len(channels), 'f')
                                                                                                                   
    for i in xrange(len(new_channels)):
      new_channels[i] = channels[self.fileIndexes[i]]
    return new_channels

  def getSpectrumRange(self, values, paramName):
    if paramName == "radiance":
      rad_scales  = jarray.zeros(self.numObsChannels, 'f')
      rad_offsets = jarray.zeros(self.numObsChannels, 'f')
      cnt = 0
      for i in xrange(self.numObsDataSets):
        System.arraycopy(self.radScale[i], 0, rad_scales, cnt, self.numChannelsPerDataSet[i])
        System.arraycopy(self.radOffset[i], 0, rad_offsets, cnt, self.numChannelsPerDataSet[i])
        cnt += self.numChannelsPerDataSet[i]
      vals1_7  = jarray.zeros(len(values), 'f')
      System.arraycopy(values, 0, vals1_7, 0, len(values))
                                                                                                                   
      vals1_7  = Helper.unpack(vals1_7, rad_scales, rad_offsets)
                                                                                                                   
      new_vals1_7  = jarray.zeros(len(values), 'f')
      for i in xrange(len(values)):
        new_vals1_7[i]  = vals1_7[self.fileIndexes[i]]

      fill = jarray.zeros(len(values), 'f')
      for i in xrange(len(values)):
        fill[i] = Float.NaN

      return [new_vals1_7, fill]
                                                                                                                   
    #- Brightness Temperature for emissive bands, Reflectance for reflective
    elif paramName == "brightnessTemp":
      rad_scales  = jarray.zeros(self.numObsChannels, 'f')
      rad_offsets = jarray.zeros(self.numObsChannels, 'f')
      ref_scales  = jarray.zeros(self.numObsChannels, 'f')
      ref_offsets = jarray.zeros(self.numObsChannels, 'f')
      cnt1 = 0
      cnt2 = 0
      for i in xrange(self.numObsDataSets):
        if self.reflScale[i] != None:
          System.arraycopy(self.reflScale[i], 0, ref_scales, cnt1, self.numChannelsPerDataSet[i])
          System.arraycopy(self.reflOffset[i], 0, ref_offsets, cnt1, self.numChannelsPerDataSet[i])
          cnt1 += self.numChannelsPerDataSet[i]
        System.arraycopy(self.radScale[i], 0, rad_scales, cnt2, self.numChannelsPerDataSet[i])
        System.arraycopy(self.radOffset[i], 0, rad_offsets, cnt2, self.numChannelsPerDataSet[i])
        cnt2 += self.numChannelsPerDataSet[i]
      vals1_7   = jarray.zeros(len(values), 'f')
      System.arraycopy(values, 0, vals1_7, 0, len(values))
                                                                                                                   
      vals1_7  = Helper.unpack(vals1_7, ref_scales, ref_offsets)
                                                                                                                   
      new_vals1_7 = jarray.zeros(len(values), 'f')
      for i in xrange(len(values)):
        new_vals1_7[i] = vals1_7[self.fileIndexes[i]]
                                                                                                                   
      fill = jarray.zeros(len(values), 'f')
      for i in xrange(len(values)):
        fill[i] = Float.NaN

      return [new_vals1_7, fill]

  def getImageRange(self, values, paramName, dataset_index, index):
    if paramName == "radiance":
      unit = Parser.parse("W/m^2/micrometer/steradian")
      self.imageRangeUnit = ScaledUnit(1, unit, "W/m^2/\xb5m/ster")
      return Helper.unpack(values, self.radScale[dataset_index][index], self.radOffset[dataset_index][index])
    elif paramName == "brightnessTemp":
       self.imageRangeUnit = Parser.parse("K")
       values = Helper.unpack(values, self.reflScale[dataset_index][index], self.reflOffset[dataset_index][index])
       return values

  def channelIndexToDataSetIndex(self, channelIndex):
    idx = self.fileIndexes[channelIndex] #- get the unsorted index
    if idx <= 1:
      return 0
    elif idx <= 6:
      return 1

  def channelIndexToFileIndex(self, datasetIndex, channelIndex):
    idx = self.fileIndexes[channelIndex]
    if datasetIndex == 0:
      return idx
    else:
      a = 0
      for i in xrange(datasetIndex):
        a += self.numChannelsPerDataSet[i]
      return idx - a
                                                                                                                   
  def makeRangeTypes(self):
    self.spectrumRangeTypeDict = {
      "radiance":
        RealTupleType(RealType.getRealType("Radiance1_7"), RealType.getRealType("Radiance")),
      "brightnessTemp":
        RealTupleType(RealType.getRealType("Reflectance"), RealType.getRealType("BrightnessTemp"))}
                                                                                                                   
    self.imageRangeTypeDict = {
      "radiance": RealType.getRealType("Radiance"),
      "brightnessTemp": RealType.getRealType("Refl/BT")}
                                                                                                                   
    self.rangeTypes = []
    self.rangeTypes.append(RealType.getRealType("Radiance1_7"))
    self.rangeTypes.append(RealType.getRealType("Radiance"))
    self.rangeTypes.append(RealType.getRealType("Reflectance"))
    self.rangeTypes.append(RealType.getRealType("BrightnessTemp"))

  def makeSpectrumRangeType(self, paramName):
    if paramName == "radiance":
      self.spectrumRangeType = RealTupleType(self.rangeTypes[0], self.rangeTypes[1])
    elif paramName == "brightnessTemp":
      self.spectrumRangeType = RealTupleType(self.rangeTypes[2], self.rangeTypes[3])

  def getObsValidRange(self):
    if self.paramName   == "radiance":
      return [[0,500], [0,10]]
    elif self.paramName == "brightnessTemp":
      return [[0,1], [100,320]]

  def makeTrackSegments(self):
    if self.numObsLines < 2600*2:
      self.numberSegments = 1
      self.trackSegments = [1, self.numObsLines]
      return
    self.numberSegments = 1
    self.trackSegments = [401*2, self.numObsLines-100*2]

  def getInitialSubset(self):
    return Subset(None,None,None,None,step=20)
 
  def clone(self):
    return ModisData500m(self.filename,self.reader,self.readerTable,master=self)

