import jarray
import struct
from util import Helper
from visad import *
from visad.util import DataUtility
from java.io import FileInputStream
from java.util import Properties
from visad.python.JPythonMethods import *
from java.lang import Float, Math, System, Object
from java.net import URL
from visad.data.units import *

from subsetImage import Subset

false = 0
true  = 1

from MultiSpectralDataSource import MultiSpectralData


class SeaDASL2Data(MultiSpectralData):
  def __init__(self, filename, reader, readerTable,master=None):
    MultiSpectralData.__init__(self, filename, reader, readerTable, master)
    self.unnavigated = true
        
    if master == None:
      reader_name  = reader.reader.__class__
      reader_name  = reader_name.__name__

  def getScaleOffset(self):
    pass

  def extractValues(self,dataToExtract):
    valuesToExtract=jarray.zeros(len(dataToExtract),'f')
    ##self.getScaleOffset()
    for i in xrange(len(dataToExtract)):
      if dataToExtract[i] < 0:
        valuesToExtract[i]=Float.NaN
      else:
        ##valuesToExtract[i]=self.paramScale[0][0]*(dataToExtract[i]-self.paramOffset[0][0])
        valuesToExtract[i]=dataToExtract[i]
    return valuesToExtract

  def makeTrackSegments(self):
    if self.numObsLines < 2600:
      self.numberSegments = 1
      self.trackSegments = [1, self.numObsLines]
      return
    self.numberSegments = 1
    self.trackSegments = [401, self.numObsLines-100]

  def makeRangeTypes(self):
    self.rangeTypes = []
    paramNames=["Chlorophyll Conc, OC2","Chlorophyll Conc, OC3","Chlorophyll-a Conc, ndpi","Chlorophyll-a Conc","Chlorophyll Conc, gsm","Chlorophyll Conc, Clark", "Chlorophyll Conc, Carder", "Total Suspended Matter"]
    self.imageRangeTypeDict={}
    for i in xrange(self.numObsDataSets):
        self.rangeTypes.append(RealType.getRealType(self.observationName[i]))
        self.imageRangeTypeDict[paramNames[i]]=RealType.getRealType(self.observationName[i])
    self.spectrumRangeTypeDict    = None

  def makeImageRangeType(self, paramName):
    self.imageRangeType = self.imageRangeTypeDict[paramName] 

  def makeSpectrumRangeType(self, paramName):
    self.spectrumRangeType = None
  
  def getImageRange(self, values):
    self.imageRangeUnit = None
    return values
    
  def getObsValidRange(self): #- valid is probably a misnomer here; mostly for setRange
     return [0, 0.18]
    
  def getNativeParam(self):
    native_param = "Chlorophyll Conc, OC2"
    return native_param

  def setDefaultParam(self):
    return self.native_param

  def changeParameter(self, paramName):
    self.paramName = paramName
  
  def getLevel2ImageSegment(self, segmentIndex=0, subset=None):
    self.fieldToRetrieve=self.imageRangeTypeDict.get(self.paramName).toString()
    self.makeImageRangeType(self.paramName)

    self.currentSegmentIndex = segmentIndex
    segStartLine = int(self.trackSegments[self.currentSegmentIndex*2]-1)  #convert one to zero based
    segEndLine   = int(self.trackSegments[self.currentSegmentIndex*2+1]-1)
    
    if subset != None: #-- subset logic
      self.subset = subset      
      if subset.end_x == None or subset.end_y == None:
        self.startElem = 0
        self.nElements = int((self.numObsElements)/subset.step)
        self.startLine = segStartLine
        self.nLines    = int((int(segEndLine-self.startLine+1))/subset.step)
      else:                 
        t_line  = subset.start_x
        t_elem  = subset.start_y
        n_lines = int((subset.end_x - subset.start_x)/subset.step) + 1
        n_elems = int((subset.end_y - subset.start_y)/subset.step) + 1
        if (self.invert == true):
          t_elem  = subset.start_x
          t_line  = subset.start_y
          n_elems = int((subset.end_x - subset.start_x)/subset.step) + 1
          n_lines = int((subset.end_y - subset.start_y)/subset.step) + 1
        
        self.startElem = t_elem
        self.startLine = segStartLine + t_line      
        self.nElements = n_elems
        self.nLines    = n_lines
      
      self.step      = self.subset.step
           
      if float(self.ratio)/float(self.step) <= 1:
        self.geo_step = int(1.0/(float(self.ratio)/float(self.step)))
        nGeoLines     = self.nLines
        nGeoElements  = self.nElements
      else:
        self.geo_step = 1
        nGeoLines     = int(float(self.nLines)/float(self.ratio)/float(self.step))
        nGeoElements  = int(float(self.nElements)/float(self.ratio)/float(self.step))
        
    else:
      self.subset    = None
      self.startElem = 0
      self.nElements = self.numObsElements
      self.startLine = segStartLine
      self.nLines    = int(segEndLine-self.startLine+1)
      
      self.step      = 1  
      
      if float(self.ratio)/float(self.step) <= 1:
        self.geo_step = int(1.0/(float(self.ratio)/float(self.step)))
        nGeoLines     = self.nLines
        nGeoElements  = self.nElements
      else:
        self.geo_step = 1
        nGeoLines     = int(float(self.nLines)/float(self.ratio)/float(self.step))
        nGeoElements  = int(float(self.nElements)/float(self.ratio)/float(self.step))
   
    start   = [self.startElem+self.geoOffset,self.startLine+self.geoOffset]
    count   = [self.nElements,self.nLines]
    stride  = [self.step, self.step]
    nstart  = [None, None]
    ncount  = [None, None]
    nstride = [None, None]
    
    for i in xrange(len(start)):
      nstart[self.obsDimOrder[i]]  = start[i]
      ncount[self.obsDimOrder[i]]  = count[i]
      nstride[self.obsDimOrder[i]] = stride[i]
      
    if self.step > 1: #- this hack will go away when Java-Netcdf implements stride reading
      self.z = self.reader.getValues(self.fieldToRetrieve, nstart, ncount, nstride)
    else:
      self.z = self.reader.getValues(self.fieldToRetrieve, nstart, ncount, None)
          
    if self.invert == true:
      self.ylength  = self.nLines
      self.xlength  = self.nElements
      self.gylength = nGeoLines
      self.gxlength = nGeoElements
    else:
      self.xlength  = self.nLines
      self.ylength  = self.nElements
      self.gxlength = nGeoLines
      self.gylength = nGeoElements

    if self.ratio == 1:
      self.gxlength = self.xlength
      self.gylength = self.ylength
     
    self.imageDomType = RealTupleType(self.spatialTupleArray[0], self.spatialTupleArray[1])
    self.imageDomain  = makeDomain(self.imageDomType,
          0,((self.xlength)/(self.xlength/self.gxlength))-1,self.xlength,
          0,((self.ylength)/(self.ylength/self.gylength))-1,self.ylength)
      
    self.imageFunctionType = FunctionType(self.imageDomType, self.imageRangeType)
    self.imageField        = FlatField(self.imageFunctionType, self.imageDomain)  
    self.z                 = self.getImageRange(self.z)
        
    self.imageField.setSamples([self.z], false)
       
    return self.imageField
      
  def clone(self):
    return SeaDASL2Data(self.filename,self.reader,self.readerTable,master=self)
