#!/usr/bin/env python
"""
$Id: hdf_jython.py,v 1.3.2.4 2007/01/22 21:17:55 brucef Exp $

Simplified test for ResamplingStage executable.

Author: R.K.Garcia, jython port: T.D.Rink
"""


#from mmap import *
from struct import *
from math import sin
from sys import stderr

import os.path
import string
import os
import operator
import glob
from exceptions import Exception
import jarray
from java.lang import String
from java.lang import System, Runtime
from java.io import DataInputStream, IOException
from java.nio import *

C_ERRS = {
    -7:"ERR_BAD_DIMID",
    -8:"ERR_BAD_DIM_INDEX",
    -9:"ERR_BAD_SDS_NAME",
    -10:"ERR_BAD_FILENAME",
    -11:"ERR_BAD_SDID",
    -12:"ERR_BAD_INDEX",
    -13:"ERR_OPEN_FAILED",
    -14:"ERR_SELECT_FAILED",
    -15:"ERR_BAD_SDSID",
    -16:"ERR_ENDACCESS_FAILED",
    -17:"ERR_END_FAILED",
    -18:"ERR_BAD_START_ARRAY",
    -19:"ERR_BAD_STRIDE_ARRAY",
    -20:"ERR_BAD_EDGES_ARRAY",
    -21:"ERR_START_STRIDE_EDGES_MISMATCH",
    -22:"ERR_READDATA_FAILED",
    -23:"ERR_BAD_OBJID", 
    -24:"ERR_BAD_ATTR_NAME",
    -25:"ERR_FINDATTR_FAILED",
    -26:"ERR_ATTRINFO_FAILED",
    -27:"ERR_BAD_ATTR_INDEX",
    -28:"ERR_READATTR_FAILED",
    -29:"ERR_UNKNOWN_ATTR_TYPE",
    -30:"ERR_NAMETOINDEX_FAILED",
    -31:"ERR_GETINFO_FAILED",
    -32:"ERR_GETDIMID_FAILED",
    -33:"ERR_GETDIMINFO_FAILED",
    -34:"ERR_FILEINFO_FAILED",
    -35:"ERR_UNKNOWN_DATA_TYPE",
    -36:"ERR_BAD_FILEID",

    -37:"ERR_BAD_VSID",
    -38:"ERR_BAD_VSNAME",
    -39:"ERR_VS_NOT_FOUND",
    -40:"ERR_SETFIELD_FAILED",
    -41:"ERR_BAD_VID",
    -42:"ERR_ATTACH_FAILED",
    -43:"ERR_BAD_VSSTART",
    -44:"ERR_BAD_VSRECS",
    -45:"ERR_INQUIRE_FAILED",
    -46:"ERR_FIELDTYPE_FAILED",
    -47:"ERR_SEEK_FAILED",

    -5150:"ERR_WTF_DUDE",
}

def get_err( code ):
    if code in C_ERRS.keys():
        return "%s(%s)" % (C_ERRS[code], code)
    else:
        return code

class SDError( Exception ):
    def __init__( self, rc ):
        self.args = ( rc, )
   
class VSError( Exception ):
    def __init__( self, rc ):
        self.args = ( rc, )

class HError( Exception ):
    def __init__( self, rc ):
        self.args = ( rc, )
  
def which( exe, *args ):
    """
    which( executable-name[, optional-pathlist[, second-pathlist...]] )
    If full path to the executable is specified, return the executable as the first member of a list.
    Optionally, further path strings can be included as preferred search locations.
    Otherwise search the system path for instances of the executable, and return a list of them in order.
    """
    if os.path.exists( exe ): return [ exe ]
    path = string.join( (list(args) or []) + [ os.environ[ 'PATH' ] ], os.pathsep )
    return reduce( operator.add, [ glob.glob( os.path.join( D, exe ) ) for D in path.split( os.pathsep ) ], [] )


def invoke( exe ):
    rt = Runtime.getRuntime()
    proc = rt.exec(exe)
    writeable = proc.getOutputStream()
    readable = proc.getInputStream()
    return writeable, readable, proc

def readStruct( (writable0, readable1, pid ), format='>i' ):
    'read a structure of known content from the pipe'
    b = jarray.zeros(calcsize(format), 'b')
    n = readable1.read(b,0,len(b))
    return unpack( format, String(b,0) )
    
def readBlock( ( writable0, readable1, pid ), kind = 'c' ):
    'read a length-first block of data from the plugin pipe'
    b = jarray.zeros(4, 'b')
    n = readable1.read(b,0,4)
    nelem = unpack( '>i', String(b,0) )[0]
    fmt = '>%d%s' % ( nelem, kind )
    b = jarray.zeros(calcsize(fmt),'b')
    n = readable1.read(b,0,len(b))
    return unpack( fmt, String(b,0))

    
def readRawBlock( ( writable0, readable1, pid ), data_type=None, element_count=None ):
    'read a length-first block of byte data from the plugin pipe'
    b = jarray.zeros(4,'b')
    n = readable1.read(b,0,4)
    nbytes = unpack('>i', String(b,0) )[0]

    b_buf = ByteBuffer.allocate(nbytes)
    bb    = b_buf.array()

    #- workaround for situation/problem seen only MAC OS X
    ntotal = 0
    while (ntotal < nbytes):
      n = readable1.read(bb, ntotal, nbytes - (ntotal))
      ntotal += n
    
    if data_type == 's':
      return bb

    #- use java.nio to format bytes
    b_buf.rewind()

    if data_type == 'I' or data_type == 'i':
      int_buf = b_buf.asIntBuffer()
      int_ary = jarray.zeros(element_count, 'i')
      int_buf.get(int_ary)
      return int_ary
    if data_type == 'H' or data_type == 'h':
      srt_buf = b_buf.asShortBuffer()
      srt_ary = jarray.zeros(element_count, 'h')
      srt_buf.get(srt_ary)
      return srt_ary
    if data_type == 'd':
      db = b_buf.asDoubleBuffer()
      da = jarray.zeros(element_count, 'd')
      db.get(da)
      return da
    if data_type == 'f':
      flt_buf = b_buf.asFloatBuffer()
      flt_ary = jarray.zeros(element_count, 'f')
      flt_buf.get(flt_ary)
      return flt_ary

    
def command( (writable0,readable1,pid), cmd, parmblock = '' ):
    writable0.write( pack( '>i', cmd ) + parmblock )
    writable0.flush()
    # read returned result code 
    b = jarray.zeros(4,'b')
    n = readable1.read(b,0,4)
    return unpack( '>i', String(b,0) )[0]

def stringBlock( str ):
    return pack( '>i', len(str) ) + str
    
def intBlock( L ):
    return pack( '>%di' % (1+len(L)), len(L), *L )
        
# command implementations per README

# module's connection to the plug-in executable
#plug_hdf = which( 'plug_hdf' )[0] 

sys_name = System.getProperty('os.name')

if sys_name.find('Linux') >= 0 or sys_name.find('linux') >= 0:
  plug_hdf = './plug_hdf'
elif sys_name.find('Windows') >= 0 or sys_name.find('windows') >= 0:
  plug_hdf = '.\plug_hdf.exe'
elif sys_name.find('Mac') >= 0:
  plug_hdf = './plug_hdf'
else:
  plug_hdf = './plug_hdf'

LINK = invoke( plug_hdf )    

def ping( ):
    return command( LINK, 0 )
    
def ident():
    rc = command( LINK, 1 )
    if rc>=0:
        str = readBlock( LINK, 's' )
        return str[0]
    return rc

def shutdown( LINK ):
    return command( LINK, -1 )


def start( filename ):
    rc = command( LINK, 2, stringBlock( filename ) )
    if rc>=0: return readStruct( LINK, '>i' )[0]
    raise SDError( rc ) # 'SDstart failed'
        
def select( sd_id, index ):
    rc = command( LINK, 3, pack( '>ii', sd_id, index ) )
    if rc>=0: return readStruct( LINK, '>i' )[0]
    raise SDError( rc ) # 'SDselect failed'

def readattr( id, index ):
    rc = command( LINK, 4, pack( '>ii', id, index ) )
    if rc>=0:
        data_type = readBlock( LINK, 's' )[0]
        elementCount = readStruct( LINK, '>i' )[0]
        return data_type, elementCount, readRawBlock( LINK, data_type, elementCount )
    raise SDError( rc ) # 'SDreadattr failed'

def findattr( id, name ):
    rc = command( LINK, 5, pack( '>i', id ) + stringBlock( name ) )
    if rc>=0: return readStruct( LINK, '>i' )[0]
    raise SDError( rc ) # 'SDfindattr failed'

def readdata( sds_id, start, stride, edges ):
    blox = intBlock( start ) + intBlock( stride ) + intBlock( edges )
    rc = command( LINK, 6, pack( '>i', sds_id ) + blox ) 
    if rc>=0:
        data_type = readBlock( LINK, 's' )[0]
        elementCount = readStruct( LINK, '>i' )[0]
        return data_type, elementCount, readRawBlock( LINK, data_type, elementCount )
    raise SDError( rc ) # 'SDreaddata failed'

        
def endaccess( sds_id ):
    return command( LINK, 7, pack( '>i', sds_id ) )
    
def nametoindex( sd_id, sds_name):
    rc = command( LINK, 8, pack( '>i', sd_id ) + stringBlock( sds_name) )
    if rc>=0: return readStruct( LINK, '>i' )[0]
    raise SDError( rc ) # 'SDnametoindex failed'

def end( sd_id ):
    return command( LINK, 9, sd_id )

def getdimid( sds_id, dim_index ):
    rc = command( LINK, 10, pack( '>ii', sds_id, dim_index) )
    if rc>=0:
        return readStruct( LINK, '>i' )[0]
    raise SDError( rc ) # 'SDgetdimid failed'

def diminfo( dim_id ):
    rc = command( LINK, 11, pack( '>i', dim_id ))
    if rc>=0:
        dim_name = readBlock( LINK, 's' )[0]
        dim_size = readStruct( LINK, '>i' )[0]
        dim_type_code = readStruct( LINK, '>i' )[0]
        dim_n_attrs   = readStruct( LINK, '>i' )[0]
        return dim_name, dim_size, dim_type_code, dim_n_attrs
    raise SDError( rc ) # 'SDdiminfo failed'

def getinfo( sds_id ):
    rc = command( LINK, 13, pack( '>i', sds_id ))
    if rc>=0:
       var_name = readBlock( LINK, 's' )[0]
       var_rank = readStruct( LINK, '>i' )[0]
       var_dim_lengths = readBlock( LINK, 'i')
       var_data_type = readStruct( LINK, '>i' )[0]
       var_num_attrs = readStruct( LINK, '>i' )[0]
       return var_name, var_rank, var_dim_lengths, var_data_type, var_num_attrs

def fileinfo( sd_id ):
    rc = command( LINK, 12, pack( '>i', sd_id ))
    if rc>=0:
        num_datasets = readStruct( LINK, '>i' )[0]
        num_global_attrs = readStruct( LINK, '>i' )[0]
        return num_datasets, num_global_attrs
    raise SDError( rc ) # 'SDfileinfo failed'

class HDFvariable: 
    def __init__( self, parent, sd_id, sds_id ):
        self.sd_id  = sd_id
        self.sds_id = sds_id
        self.parent = parent

    def readattr( self, index ):
        return readattr( self.sds_id, index )
        
    def findattr( self, name ):
        return findattr( self.sds_id, name )
        
    def readdata( self, start, stride, edges ):
        return readdata( self.sds_id, start, stride, edges )

    def getdimid( self, dim_index ):
        return getdimid(self.sds_id, dim_index)

    def getinfo( self ):
        return getinfo(self.sds_id)
        
    def id(self):
        return self.sds_id

    def __repr__(self):
        return "HDF variable object %d (child of %s)" % (self.sds_id, `self.parent`)
        
        
    
class HDFfile:

    def __init__( self, filename ):
        self.sd_id = start( filename )

    def select( self, index ):
        return HDFvariable( self, self.sd_id, select( self.sd_id, index ) )
        
    def readattr( self, index ):
        return readattr( self.sd_id, index )
        
    def findattr( self, name ):
        return findattr( self.sd_id, name )

    def nametoindex( self, sds_name ):
        return nametoindex( self.sd_id, sds_name )

    def diminfo( self, dim_id ):
        return diminfo( dim_id )

    def fileinfo( self ):
        return fileinfo( self.sd_id )
        
    def id( self ): 
        return self.sd_id
        
    def __repr__(self):
        return "HDF file object %d" % self.sd_id







#
# VDATA CODE
#

def h_open( filename ):
    rc = command( LINK, 14, stringBlock( filename ) )
    if rc>=0: return readStruct( LINK, 'i' )[0]
    raise HError( rc ) # Hopen failed

def h_close( f_id ):
    rc = command( LINK, 15, pack( '>i', f_id ) );
    if rc>=0: return readStruct( LINK, 'i' )[0]
    raise HError( rc ) # Hclose failed
   
def v_start( f_id ):
    """return 0 on success, err code otherwise.
    """
    rc = command( LINK, 16, pack( '>i', f_id ) )
    if rc>=0: return readStruct( LINK, 'i' )[0]
    raise VSError( rc ) # Vstart failed

def vs_find( f_id, data_name ):
    """f_id - file id returned from v_start
    data_name - Vdata string name
    return Vdata ref number if found, 0 otherwise.
    """
    rc = command( LINK, 17, pack('>i', f_id) + stringBlock( data_name ) )
    if rc>=0: return readStruct( LINK, 'i' )[0]
    raise VSError( rc ) # VSfind failed

def v_end( v_id ):
    rc = command( LINK, 18, pack('>i', v_id) )
    if rc>=0: return readStruct( LINK, 'i' )[0]
    raise VSError( rc )
  
def vs_attach( f_id, v_id ):
    rc = command( LINK, 20, pack('>ii', f_id, v_id) )
    if rc>=0: return readStruct( LINK, 'i' )[0]
    raise VSError( rc )
  
def vs_detach( v_id ):
    rc = command( LINK, 21, pack( '>i', v_id ) )
    if rc>=0: return readStruct( LINK, 'i' )[0]
    raise VSError( rc )

def vs_read( v_id, name, start_idx, n_recs, stride ):
    rc = command( 
        LINK, 19, 
        pack('>i', v_id ) + 
        stringBlock( name ) + 
        pack('>3i', start_idx, n_recs, stride)
    )
    if rc>=0:
        data_type = readBlock( LINK, 's' )[0]
        elementCount = readStruct( LINK, 'i' )[0]
        return data_type, elementCount, readRawBlock( LINK, data_type, elementCount )
    raise VSError( get_err(rc) ) # 'Vread failed'

class HDFVdata: 
    def __init__( self, parent, name, f_id, v_ref ):
        self.f_id = f_id
        self.v_ref = v_ref
        self.parent = parent
        self.name = name
        rc = vs_attach( self.f_id, self.v_ref )
        if rc == -1: raise VSError( rc )
        self.v_id = rc
        """
        print "HDFVdata(parent:'%s' name:%s f_id:%s v_id:%s) v_ref:%s" % \
                    (parent, name, f_id, self.v_id, self.v_ref)
        """

    def read( self, n_recs=-1, start_idx=0, stride=1 ):
        #print "HDFVdata.read(n_recs: %s start_idx: %s)" %(n_recs, start_idx)
        return vs_read( self.v_id, self.name, start_idx, n_recs, stride )

    def id(self):
        return self.v_id

    def __repr__(self):
        return "HDF Vdata object id %d (child of %s)" % (self.v_id, `self.parent`)

    """
    def __del__(self):
        vs_detach( self.v_id )
        v_end( self.f_id )
    """

class HDFVfile:
    def __init__( self, filename ):
        """filename - hdf filename
        """
        self.f_id = h_open( filename )
        rc = v_start( self.f_id )
        if rc == 0: raise VSError( rc )

    def find( self, data_name ):
        """Find a Vdata object based on name.
        Assumes there is only 1 field with the same name as the Vdata.
        
        data_name - name of Vdata object.
        
        return Vdata reference number
        """
        return vs_find( self.f_id, data_name )

    def select( self, name ):
        """Select a Vdata object named name. Assumes there is only 1 field 
        with the same name as the Vdata.
        
        name - name of the Vdata object
        
        return HDFVdata object
        """
        v_ref = self.find( name )
        return HDFVdata( self, name, self.f_id, v_ref )

    def id( self ):
        """return the file id as returned from Hopen
        """
        return self.f_id

    def __repr__( self ):
        return "HDF V file object: %s" % self.f_id

    """
    def __del__( self ):
        rc = h_close( self.f_id )
        if rc<0: print "Hclose failed for", self.f_id
    """







#
# TEST CODE
#

def test1():
    print "forking executable stage.." 
    wrp = invoke( './plug_hdf' )
    print "ping result: %s" % ping( wrp )
    print "ident result: %s" % ident( wrp )
    #print "resample result: %s" % resample( wrp )
    print "exit request result: %s" % doexit( wrp )
    print "waiting for exit.." 
    os.waitpid( wrp[2], 0 )
    print "done."

def test2( args ):
    f = HDFfile( args[0] )
    file = f
    file_info        = file.fileinfo()
    num_datasets     = file_info[0]
    num_global_attrs = file_info[1]


    #- set-up dictionarys for variables/dimensions
    vars = {}
    dims = {}
    vv   = []

    for i in xrange(num_datasets):  # loop through all variables
      vv.append(file.select(i))

    for i in xrange(num_datasets):  # loop through all variables
      var = vv[i]
      info =  var.getinfo()
      dim_names = []
      dim_lengths = []
      for j in xrange(info[1]):
        dim_id = var.getdimid(j)
        dim_info = file.diminfo(dim_id)
        dims[dim_info[0]] = dim_info[1]
        dim_names.append(dim_info[0])
        dim_lengths.append(dim_info[1])
      vars[info[0]] = [var, dim_names, dim_lengths]


    print vars
    print '-------------------------'
    print dims

    print f
    print f.fileinfo()
    print "-----"

    sds_name = "EV_1KM_Emissive"
    sds_idx = f.nametoindex(sds_name)
    v = f.select(sds_idx)
    print v
    print v.getinfo()
    d_id =  v.getdimid(0)
    print String((f.readattr(0))[2])
    print v.readattr(0)

    #-- read multi array (everything)
    var = vars[sds_name][0]
    start  = []
    stride = []
    size   = []
    dim_lengths = vars[sds_name][2]
    rank = len(dim_lengths)
    for i in xrange(rank):
      start.append(0)
      stride.append(1)
      size.append(dim_lengths[i])
    print start, stride, size

    try:
      print System.currentTimeMillis()
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      rr = v.readdata([0,0,0],[1,1,1],[1,2000,1200])
      print System.currentTimeMillis()
    except Exception, e:
      print e

def test3( args ):
    """
    args[0] - filename
    args[1] - VData name
    args[2] - start record
    args[3] - number of records
    """
    f_sd = HDFfile( args[0] )
    sds_name = "Radar_Reflectivity"
    sds_idx = f_sd.nametoindex(sds_name)
    v_sd = f_sd.select(sds_idx)
    print v_sd
    print v_sd.getinfo()

    f = HDFVfile( args[0] )
    v = f.select( args[1] )
    start = len(args) > 2 and int(args[2]) or 0
    n_recs = len(args) > 3 and int(args[3]) or -1
    stride = len(args) > 4 and int(args[4]) or 1
    print "start:", start, "n_recs:", n_recs, "stride:", stride
    r = v.read(start_idx=start, n_recs=n_recs, stride=stride)
    print r
    print '-------------------'
    vals = v_sd.readdata([start, start], [stride, stride], [n_recs, n_recs])
    print vals[2]


if __name__=='__main__':
    from sys import argv
    tests = { 
        '1':test1,
        '2':test2,
        '3':test3
    }
    tests[argv[1]](argv[2:])


