import jarray
import java
from javax.swing import *
from javax.swing.border import LineBorder, CompoundBorder, EmptyBorder
from java.awt import GridLayout, BorderLayout, FlowLayout, Color, Dimension, Canvas
from java.awt.event import InputEvent
import subs, graph
from subs import *
from graph import *
from visad.python.JPythonMethods import *
from visad import *
from visad.java3d import DirectManipulationRendererJ3D
from visad.bom import PickManipulationRendererJ3D, RubberBandBoxRendererJ3D, CurveManipulationRendererJ3D, HydraImageRendererJ3D
from visad.util import Delay
from ui.colorbar import ColorBarWidget, ColorBar
from java.util import StringTokenizer
from java.lang import Float, Thread, Integer, Math, Runtime
from util import Helper, ValueHistogram, OneDProbeRendererJ3D, MyRubberBandBoxRendererJ3D
from histostretch_3chan import make3channelHistogram
from DisplayViewSelect import DisplayViewSelectToolBar, HAction

import hydra_properties

#---------------------------------------
false = 0
true  = 1
lightWeightPopupEnabled = false
#---------------------------------------

selector_colors = [Color.magenta, Color(0.5, 0.5, 0.0), Color(0.0, 0.5, 0.5), Color(1.0, 0.0, 0.0), Color(0.0, 0.0, 1.0)]

runtime = Runtime.getRuntime()
red_ch = RealType.getRealType("red_ch")
green_ch = RealType.getRealType("green_ch")
blue_ch = RealType.getRealType("blue_ch")

import hydra_properties
import hydra_globals
from hydra_globals import getImageRenderer, hydra_path, geoPointRef_s
from hframe import HFrame

import lockCell
from lockCell import LockCell


class RGB_Combine:
  def __init__(self, caller, selectDisplay=true):
    self.filereader  = caller.ad
    self.operPair    = None
    self.spectrum    = caller.spectrum
    self.obs_valid_range = self.filereader.getObsValidRange()
    self.displayLonType = caller.displayLonType
    self.displayLatType = caller.displayLatType
    self.displayAltType = caller.displayAltType
    self.ds_cs          = caller.ds_cs
    self.lon_low        = caller.lon_low
    self.lon_hi         = caller.lon_hi
    self.lat_low        = caller.lat_low
    self.lat_hi         = caller.lat_hi
    self.red_minmax     = None
    self.grn_minmax     = None
    self.blu_minmax     = None
    self.bluBandName    = None
    self.redBandName    = None
    self.grnBandName    = None
    
    if (selectDisplay == true):
      self.makeSelectDisplay()
    
    self.resultX            = None
    self.resultY            = None
    self.resultDisplayList  = [] 
    
  def windowClose(self, event):
    rlen = len(self.resultDisplayList)
    for i in range(rlen):
      self.resultDisplayList[0].windowClose(event)
          
  def getOperandData(self, value):
    spectralIndex = self.filereader.observationDomain.valueToIndex([[value]])[0]
    return self.filereader.getSameImageSegmentCopy(spectralIndex)
    
  def compute(self, event):
    result = self.operPair.evaluate()
    ResultDisplay(self, result)

  def computeFromFields(self, red=None, grn=None, blu=None):
    result = self.evaluateFromFields(red,grn,blu)
    ResultDisplay(self, result)

  def makeSelectDisplay(self):
    #-- VisAD displays are awt.Canvas -> Canvas3D, so mixing heavyweight/lightweight 
    #-- components causes problems with the popup menus always rendered beneath the Canvas
    JPopupMenu.setDefaultLightWeightPopupEnabled(lightWeightPopupEnabled)

    frame     = HFrame("RGB Combine Tool", windowClosing=self.windowClose)
    framePane = frame.getContentPane()
    framePane.setLayout(BorderLayout())
           
    global spectrumDisplay
    obsElementInstrument, obsLineInstrument, obsChannelIndexInstrument, observationChannelsInstrument, observationsInstrument = self.filereader.getRealTypes()
    rtype = self.filereader.spectrumRangeType
    
    dset    = self.filereader.observationDomain
    self.lo = dset.getLow()
    self.hi = dset.getHi()
    lens    = dset.getLengths()
    spmps = None
    ylohi = None
    
    if isinstance(rtype, RealTupleType):
      spmps = makeMaps(observationChannelsInstrument,'x',rtype.getComponent(0),'y',rtype.getComponent(1),'y')
      spmps[0].setRange(self.lo[0], self.hi[0])
      spmps[1].setRange(self.obs_valid_range[0][0], self.obs_valid_range[0][1])
      spmps[2].setRange(self.obs_valid_range[1][0], self.obs_valid_range[1][1])
      as = spmps[2].getAxisScale()
      as.setVisible(1)
      as.setSide(1)
      as.setColor(Color.magenta)
      ylohi = self.obs_valid_range[1]
      observationsInstrument = rtype.getComponent(1)
      self.ymap2 = spmps[2]
    else:
      spmps = makeMaps(observationChannelsInstrument,'x',observationsInstrument,'y')
      spmps[1].setRange(self.obs_valid_range[0], self.obs_valid_range[1])
      spmps[0].setRange(self.lo[0], self.hi[0])
      ylohi = self.obs_valid_range
    
    spectrumDisplay = makeDisplay(spmps)
    subs.setAspectRatio(spectrumDisplay,2)
    mode = spectrumDisplay.getGraphicsModeControl()
    mode.setPointSize(5)
    showAxesScales(spectrumDisplay,1)
    setBoxSize(spectrumDisplay,.6,clip=0)
    
    dsp_rdr = spectrumDisplay.getDisplayRenderer()
    dsp_rdr.setBackgroundColor(hydra_properties.background_color)
    dsp_rdr.setForegroundColor(hydra_properties.foreground_color)

    if isinstance(rtype, RealTupleType):
      addData('b',self.spectrum.getComponent(0),spectrumDisplay,subs.makeColorMap(hydra_properties.foreground_color),zlayer=-0.05) 
      addData('b',self.spectrum.getComponent(1),spectrumDisplay,subs.makeColorMap("magenta"),zlayer=-0.05) 
    else:
      addData('b',self.spectrum,spectrumDisplay,subs.makeColorMap(hydra_properties.foreground_color), zlayer=-0.05) 
    #---------------------------
    
    self.pcntrl = spectrumDisplay.getProjectionControl()
    self.imtrx  = self.pcntrl.getMatrix()
    self.xmap   = spmps[0]
    self.ymap   = spmps[1]


    global rval, redSelectRef, gval, greenSelectRef, bval, blueSelectRef
    if self.filereader.instrumentName == 'MODIS':
      rval = self.filereader.bandNameToValue('1')
      gval = self.filereader.bandNameToValue('4')
      bval = self.filereader.bandNameToValue('3')
      redInit   = [rval]
      greenInit = [gval]
      blueInit  = [bval]
    else:
      rval     = int(lens[0]*0.25)
      redInit = self.filereader.observationDomain.indexToValue([rval])[0]
      gval     = int(lens[0]*0.50)
      greenInit = self.filereader.observationDomain.indexToValue([gval])[0]
      bval = int(lens[0]*0.75)
      blueInit = self.filereader.observationDomain.indexToValue([bval])[0]

    redSelectRef = spectrumDisplay.addData("red", Real(observationChannelsInstrument, redInit[0]),
      constantMaps=[ConstantMap(-0.5, Display.YAxis)]+makeColorMap("red"), renderer=OneDProbeRendererJ3D(dset))
    greenSelectRef = spectrumDisplay.addData("green", Real(observationChannelsInstrument, greenInit[0]),
      constantMaps=[ConstantMap(-0.5, Display.YAxis)]+makeColorMap("green"), renderer=OneDProbeRendererJ3D(dset))
    blueSelectRef = spectrumDisplay.addData("blue", Real(observationChannelsInstrument, bval),
      constantMaps=[ConstantMap(-0.5, Display.YAxis)]+makeColorMap("blue"), renderer=OneDProbeRendererJ3D(dset))
      
    global lpair, rpair
    if self.filereader.instrumentName == "MODIS" or self.filereader.instrumentName == "MSG" or self.filereader.instrumentName == "GOES-E" or self.filereader.instrumentName == "GOES-W" or self.filereader.instrumentName == "MODIS_500m" or self.filereader.instrumentName == "GOES-10":
      self.operPair = OperationPairPullDown(redInit[0], redSelectRef, greenInit[0], greenSelectRef, blueInit[0], blueSelectRef, self)
    else:
      self.operPair = OperationPair(redInit[0], redSelectRef, greenInit[0], greenSelectRef, blueInit[0], blueSelectRef, self)
      
    self.operPair.getComponent().setMaximumSize(self.operPair.getComponent().getPreferredSize())
    
    computeButton = JButton("Compute", actionPerformed=self.compute)
    sp = JPanel()
    sp.setLayout(FlowLayout())
    sp.add(computeButton)
    sp.setMaximumSize(sp.getPreferredSize())
    
    tb = DisplayViewSelectToolBar([spectrumDisplay], [1], pickAction="grab")
    tb.add(sp)

    framePane.add(tb, BorderLayout.NORTH)
    framePane.add(spectrumDisplay.getComponent(), BorderLayout.CENTER)
    framePane.add(self.operPair.getComponent(), BorderLayout.SOUTH)

    class RedCell(CellImpl):
      def __init__(self, operPair, obs_valid_range):
        self.operPair = operPair
        self.obs_valid_range = obs_valid_range
        self.lineRefL = spectrumDisplay.drawLine([[rval, rval], [self.obs_valid_range[0], self.obs_valid_range[1]]], color="red", width=2)
        
      def doAction(self):
        val = redSelectRef.getData().getValue()
        spectrumDisplay.moveLine(self.lineRefL, [[val, val], [self.obs_valid_range[0], self.obs_valid_range[1]]])
        self.operPair.lockout(1)
        self.operPair.setRed('%8.3f' % val)
        self.operPair.lockout(0)
    rcell = RedCell(self.operPair, ylohi)
    rcell.addReference(redSelectRef)

    class GreenCell(CellImpl):
      def __init__(self, operPair, obs_valid_range):
        self.operPair = operPair
        self.obs_valid_range = obs_valid_range
        self.lineRefR = spectrumDisplay.drawLine([[gval, gval], [self.obs_valid_range[0], self.obs_valid_range[1]]], color="green", width=2)
        
      def doAction(self):
        val = greenSelectRef.getData().getValue()
        spectrumDisplay.moveLine(self.lineRefR, [[val, val], [self.obs_valid_range[0], self.obs_valid_range[1]]])    
        self.operPair.lockout(1)
        self.operPair.setGreen('%8.3f' % val)
        self.operPair.lockout(0)
    gcell = GreenCell(self.operPair, ylohi)
    gcell.addReference(greenSelectRef)

    class BlueCell(CellImpl):
      def __init__(self, operPair, obs_valid_range):
        self.operPair = operPair
        self.obs_valid_range = obs_valid_range
        self.lineRefL = spectrumDisplay.drawLine([[bval, bval], [self.obs_valid_range[0], self.obs_valid_range[1]]], color="blue", width=2)
        
      def doAction(self):
        val = blueSelectRef.getData().getValue()
        spectrumDisplay.moveLine(self.lineRefL, [[val, val], [self.obs_valid_range[0], self.obs_valid_range[1]]]) 
        self.operPair.lockout(1)
        self.operPair.setBlue('%8.3f' % val)
        self.operPair.lockout(0)
    bcell = BlueCell(self.operPair, ylohi)
    bcell.addReference(blueSelectRef)
    
    frame.pack()
    frame.show()
    frame.validate()
    frame.addVisADDisplay(spectrumDisplay)
#---------------------------------------- makeSelectDisplay

  def evaluateFromFields(self, rdata, gdata, bdata):
    type = rdata.getType()
    new_type = FunctionType(type.getDomain(), red_ch)
    dset = rdata.getDomainSet()
    values     = Helper.unpackFloats(rdata,false)
    self.rdata = FlatField(new_type, dset)
    self.rdata.setSamples(values, false)
                                                                                                                               
    type = gdata.getType()
    new_type = FunctionType(type.getDomain(), green_ch)
    dset = gdata.getDomainSet()
    values     = Helper.unpackFloats(gdata,false)
    self.gdata = FlatField(new_type, dset)
    self.gdata.setSamples(values, false)
                                                                                                                               
    type = bdata.getType()
    new_type = FunctionType(type.getDomain(), blue_ch)
    dset = bdata.getDomainSet()
    values     = Helper.unpackFloats(bdata,false)
    self.bdata = FlatField(new_type, dset)
    self.bdata.setSamples(values, false)
    result = FieldImpl.combine([self.rdata, self.gdata, self.bdata])
                                                                                                                               
    return result


class ResultDisplay:
  count           = 0
  geo_map = load(hydra_path+'/data/outlsupw')
  
  def __init__(self, caller, result):
    runtime.gc()
    
    self.dirty      = false
    self.caller     = caller
    self.result     = result
    self.text = TextType.Generic
    self.n_selectors   = 5
    self.select_index  = 0
    self.boxSelected   = true
    self.curveSelected = false
    self.ds_cs         = caller.ds_cs
    
    obsElementInstrument, obsLineInstrument, obsChannelIndexInstrument, observationChannelsInstrument, observationsInstrument = self.caller.filereader.getRealTypes()
    obsLatitudeInstrument, obsLongitudeInstrument = self.caller.filereader.getGeoRealTypes()
    obsLatMinMaxInstrument, obsLonMinMaxInstrument = self.caller.filereader.getGeoDomain()    
    observationChannelsInstrumentMX = self.caller.filereader.getObservationChannelRange()

    if (self.caller.filereader.instrumentName == "MODIS" or self.caller.filereader.instrumentName == "MODIS_500m")  and self.caller.redBandName == '1' and self.caller.grnBandName == '4' and self.caller.bluBandName == '3':
      maps = makeMaps(obsLongitudeInstrument,'x', obsLatitudeInstrument, 'y', red_ch,'rgb',green_ch,'rgb',blue_ch,'rgb','marker','shape', self.text, 'text', RealType.XAxis, 'x', RealType.YAxis, 'y')
      if caller.displayLonType != None and caller.displayLatType != None:
        maps[0] = ScalarMap(obsLongitudeInstrument, caller.displayLonType)
        maps[1] = ScalarMap(obsLatitudeInstrument, caller.displayLatType)

      if self.caller.filereader.paramName == 'brightnessTemp':
        if self.caller.red_minmax[1] > 1.0:
          maps[2].setRange(self.caller.red_minmax[0], 1.0)
        if self.caller.grn_minmax[1] > 1.0:
          maps[3].setRange(self.caller.grn_minmax[0], 1.0)
        if self.caller.blu_minmax[1] > 1.0:
          maps[4].setRange(self.caller.blu_minmax[0], 1.0)

      # Liam's enhancement - use 200 steps
      ct = [ (90. * i /19.)/255. for i in xrange(0,20)] + \
       [(90.+(210.-90.)*(float(i)-20.)/(90.-20.))/255. for i in xrange(20,90)] + \
       [(210.+(255.-210.)*(float(i)-90.)/(200.-90.))/255. for i in xrange(90,200)]
                                                                                                                           
      self.dsp = makeDisplay(maps)
      mode = self.dsp.getGraphicsModeControl()
      mode.setColorMode(1)
      zt = [0. for i in ct]
      maps[2].getControl().setTable( (ct,zt,zt))
      maps[3].getControl().setTable( (zt,ct,zt))
      maps[4].getControl().setTable( (zt,zt,ct))
      self.maps = maps
    else:
      maps = makeMaps(obsLongitudeInstrument,'x', obsLatitudeInstrument, 'y', red_ch,'red',green_ch,'green',blue_ch,'blue','marker','shape', self.text, 'text', RealType.XAxis, 'x', RealType.YAxis, 'y')
      if caller.displayLonType != None and caller.displayLatType != None:
        maps[0] = ScalarMap(obsLongitudeInstrument, caller.displayLonType)
        maps[1] = ScalarMap(obsLatitudeInstrument, caller.displayLatType)
      self.dsp = makeDisplay(maps)
      self.maps = maps

    self.maps[0].setRange(caller.lon_low, caller.lon_hi)
    self.maps[1].setRange(caller.lat_low, caller.lat_hi)
    self.maps[7].setRange(-1, 1)
    self.maps[8].setRange(-1, 1)
    setBoxSize(self.dsp,.7,clip=0)

    mode = self.dsp.getGraphicsModeControl()
    dsp_rdr = self.dsp.getDisplayRenderer()
    dsp_rdr.setBackgroundColor(hydra_properties.background_color)
    dsp_rdr.setForegroundColor(hydra_properties.foreground_color)
    
    dsp_rdr.setCursorStringOn(false)
    self.mHelper = dsp_rdr.getMouseBehavior().getMouseHelper()
    self.mHelper.setFunctionMap([[[MouseHelper.DIRECT, MouseHelper.ZOOM],
                                   [MouseHelper.TRANSLATE, MouseHelper.NONE]],
                                  [[MouseHelper.CURSOR_TRANSLATE, MouseHelper.CURSOR_ZOOM],
                                   [MouseHelper.CURSOR_ROTATE, MouseHelper.NONE]],
                                  [[MouseHelper.DIRECT, MouseHelper.DIRECT],
                                   [MouseHelper.DIRECT, MouseHelper.DIRECT]]])
    
    ##self.resultRef     = self.dsp.addData('b', result, constantMaps=[ConstantMap(-0.1, caller.displayAltType)],renderer=getImageRenderer())
    self.resultRef     = self.dsp.addData('b', result, constantMaps=[ConstantMap(-0.1, caller.displayAltType)])
    self.mapRef = self.dsp.addData('m',ResultDisplay.geo_map,constantMaps=subs.makeColorMap(hydra_properties.foreground_color)+[ConstantMap(-0.05, caller.displayAltType)])

    mode.setTextureEnable(hydra_globals.textureEnable)

    self.shapes = None
    self.shapes = subs.Shapes(self.dsp,self.maps[5])
    self.shapeMarkerIndex = None
    self.shapeMarkerIndex = self.shapes.addShape("cross",scale=.03,color="cyan", autoScale=0)
    ll = self.caller.filereader.cs.toReference([[0], [0]])
    if self.ds_cs != None:
      ll = self.ds_cs.toReference([[ll[1][0]], [ll[0][0]], [0]])
    self.shapes.moveShape(self.shapeMarkerIndex, (ll[0][0], ll[1][0]))

    txtCntrl = self.maps[6].getControl()
    txtCntrl.setJustification(TextControl.Justification.LEFT)
    txtCntrl.setAutoSize(1)
    txtCntrl.setFont(Font("sansserif", Font.PLAIN, 16))
    
    #-- string label stuff
    """
    operString = self.caller.operPair.toString()
    st = StringTokenizer(operString, "()+-/*")
    channelVals = []
    ns = operString
    chStr = ""
    for i in xrange(st.countTokens()):
      if chStr != "":
        chStr = chStr + ',  '
      tok = st.nextElement()
      channelVals.append(tok)
      tt = 'c%i' % (i+1)
      ns = ns.replace(tok, tt)
      chStr = chStr + tt
      chStr = chStr + ":"+tok
    """

    class Pick:
      def __init__(self, dsp, shapes, shapeMarkerIndex, ds_cs, updateGeoPoint, xmap=None, ymap=None):
        self.dsp      = dsp
        self.shapes   = shapes
        self.shapeMarkerIndex = shapeMarkerIndex
        self.ds_cs    = ds_cs
        subs.HandlePickEvent(self.dsp, self.doAction, xmap, ymap, None)
        self.updateGeoPoint = updateGeoPoint
                                                                                                                                      
      def doAction(self, x, y):
        self.shapes.moveShape(self.shapeMarkerIndex,(x,y))
        if self.ds_cs != None:
          ll = self.ds_cs.fromReference([[x], [y], [0]])
          lon = ll[0][0]
          lat = ll[1][0]
        else:
          lon = x
          lat = y
        self.updateGeoPoint(lon,lat,0,0)
                                                                                                                                      
    #- use the copy so we can get the non-masked field values
    pick = Pick(self.dsp, self.shapes, self.shapeMarkerIndex, self.ds_cs, self.updateGeoPoint,self.maps[7], self.maps[8])

    #frame = HFrame("Linear Combination: "+ns+"  "+chStr,windowClosing=self.windowClose)
    frame = HFrame("RGB",windowClosing=self.windowClose)
    
    #- menuBar
    menuBar     = JMenuBar()
    toolMenu    = JMenu("Tools")
    settingsMenu = JMenu("Settings")
    menuBar.add(toolMenu)
    menuBar.add(settingsMenu)
    item1 = JMenuItem("Capture Display", actionPerformed=self.captureDisplay)
    item1.setEnabled(1)
    toolMenu.add(item1)
    item1 = JMenuItem("Set Color Range", actionPerformed=self.doSetColorRange)
    item1.setEnabled(1)
    settingsMenu.add(item1)
    frame.setJMenuBar(menuBar)
    self.frame = frame

    lonlatgridMenu = JMenu("Lon/Lat Grid")
    item = JRadioButtonMenuItem("enabled", actionPerformed=self.enableLonLatGrid)
    item.setSelected(0)
    item.setEnabled(1)
    lonlatgridMenu.add(item)
    settingsMenu.add(lonlatgridMenu)

    imgLblMenu = JMenu("Image Label")
    item = JRadioButtonMenuItem("enabled", actionPerformed=self.enableImageLabel)
    item.setSelected(0)
    item.setEnabled(1)
    imgLblMenu.add(item)
    settingsMenu.add(imgLblMenu)


    framePane  = frame.getContentPane()
    framePane.setLayout(BorderLayout())
    framePane.add(self.dsp.getComponent(), BorderLayout.CENTER)
    textPanel = JPanel()
    textLabel = JLabel()
    textLabel.setForeground(hydra_properties.foreground_color)
    textPanel.setBackground(hydra_properties.background_color)
    self.dsp.setAlwaysAutoScale(false)
    
    frame.setSize(500,500)
    frame.pack()
    frame.show()
    frame.addVisADDisplay(self.dsp)
    
    self.caller.resultDisplayList.append(self)
    ResultDisplay.count += 1

    from lonlatgrid import lonlatgrid
    #self.ll_grid = lonlatgrid(self.dsp, self.ds_cs, self.colorMenu)
    from imagelabel import ImageLabel
    ad = self.caller.filereader
    #self.imageLabel = ImageLabel(self.dsp, self.ds_cs, None, ad.instrumentName,ad.date_str,ad.time_str, obsLongitudeInstrument, obsLatitudeInstrument, self.colorMenu)

    self.geoPtCell = LockCell(geoPointRef_s[0], self.updateProbe)
    self.geoPtCell.addReference(geoPointRef_s[0])




  def updateProbe(self,lonlatTuple):
    self.updateMarker(lonlatTuple)

  def updateMarker(self,lonlatTuple):
    if lonlatTuple == None:
      return
    comps = lonlatTuple.getComponents()
    lon = comps[0].getValue()
    lat = comps[1].getValue()
    x = lon
    y = lat
    if self.ds_cs != None:
      xyz = self.ds_cs.toReference([[lon],[lat],[0]])
      x = xyz[0][0]
      y = xyz[1][0]
    
    self.shapes.moveShape(self.shapeMarkerIndex,(x,y))

  def updateGeoPoint(self, lon, lat, idx, noTrigger=false):
    if noTrigger == true:
      self.geoPtCell.setNoTrigger(true)
    geoPointRef_s[idx].setData(RealTuple(RealTupleType.SpatialEarth2DTuple, [lon, lat]))


  def enableLonLatGrid(self, event):
    if event.getSource().isSelected() == true:
      self.ll_grid.enable(self.ds_cs)
    elif event.getSource().isSelected() == false:
      self.ll_grid.disable()

  def enableImageLabel(self,event):
    if event.getSource().isSelected() == true:
      self.imageLabel.enable(self.ds_cs)
    elif event.getSource().isSelected() == false:
      self.imageLabel.disable()

  def setSelectedIndex(self,event):
    cmd = event.getActionCommand()
    self.select_index = int(cmd)
    for i in xrange(self.n_selectors):
      if i == self.select_index:
        if self.boxSelected == true:
          self.rbbrj3d_s[i].active = true
          self.dsp.toggle(self.crvRef_s[i], false)
        if self.curveSelected == true:
          self.dsp.toggle(self.crvRef_s[i], true)
          self.rbbrj3d_s[i].active = false
      else:
        self.rbbrj3d_s[i].active = false
        self.dsp.toggle(self.crvRef_s[i], false)
    
  def selectBox(self, event):
    self.boxSelected   = true
    self.curveSelected = false
    for i in xrange(self.n_selectors):
      self.dsp.toggle(self.crvRef_s[i], false)
    self.rbbrj3d_s[self.select_index].active = true
    
  def selectCurve(self, event):
    self.boxSelected   = false
    self.curveSelected = true
    for i in xrange(self.n_selectors):
      self.rbbrj3d_s[i].active = false
    self.dsp.toggle(self.crvRef_s[self.select_index], true)
  
  def selectX(self, event):
    self_idx = self.caller.resultDisplayList.index(self)
    for i in range(len(self.caller.resultDisplayList)):
      rr = self.caller.resultDisplayList[i]
      if (i != self_idx):        
        self.caller.resultDisplayList[i].xb.setSelected(false)
      else:
        if rr.yb.isSelected():
          rr.yb.setSelected(false)
          self.caller.setY(None)
    self.caller.setX(self)
    
  def selectY(self, event):
    self_idx = self.caller.resultDisplayList.index(self)
    for i in range(len(self.caller.resultDisplayList)):
      rr = self.caller.resultDisplayList[i]
      if (i != self_idx):
        self.caller.resultDisplayList[i].yb.setSelected(false)
      else:
        if rr.xb.isSelected():
          rr.xb.setSelected(false)
          self.caller.setX(None)
    self.caller.setY(self)
    
  def toggleMask(self, event):
    if (self.togIndex == 0):
      self.mask_vals = Helper.unpackFloats(self.result, false)
      self.result.setSamples(self.values, false)
    if (self.togIndex == 1):
      self.result.setSamples(self.mask_vals, false)
    self.mskToggleBttn.setText(self.togText[self.lastTogIndex])
    tmp = self.togIndex
    self.togIndex = self.lastTogIndex
    self.lastTogIndex = tmp
    
  def doSetColorRange(self,event):
    make3channelHistogram([self.maps[2],self.maps[3], self.maps[4]], self.result)
    
  def captureDisplay(self,event):
    Helper.saveDisplay(self.dsp, JFrame().getContentPane())
    
  def clear(self, select_index):
    self.crvRef_s[select_index].setData(UnionSet([self.noSet]))
    self.last_crvRef_s[select_index].setData(UnionSet([self.noSet]))
    self.rbbrj3d_s[select_index].removeLastBox()
  
  def boxUpdate(self,line,elem):
    for i in range(len(self.caller.scatterDisplayList)):
      self.caller.scatterDisplayList[i].markScatterPoints(line, elem, self.select_index)
      #- Retain contents of the last box
      self_idx = self.caller.resultDisplayList.index(self)
      for i in range(len(self.caller.resultDisplayList)):
        self.caller.resultDisplayList[i].crvRef_s[self.select_index].setData(UnionSet([self.caller.resultDisplayList[i].noSet]))
        self.caller.resultDisplayList[i].last_crvRef_s[self.select_index].setData(UnionSet([self.caller.resultDisplayList[i].noSet]))
        if (i != self_idx):
          self.caller.resultDisplayList[i].rbbrj3d_s[self.select_index].setLastBox(self.caller.resultDisplayList[self_idx].rbbrj3d_s[self.select_index])  
                    
  def subsetByCurve(self, event):
    if ((event.getId() == DisplayEvent.MOUSE_RELEASED_RIGHT or event.getId() == DisplayEvent.MOUSE_RELEASED_LEFT) and self.curveSelected == true):
      uset  = self.crvRef_s[self.select_index].getData()
      sets  = uset.getSets()
      s_idx = len(sets) - 1
      self.crvRef_s[self.select_index].setData(UnionSet([sets[s_idx]]))
      self.last_crvRef_s[self.select_index].setData(UnionSet([sets[s_idx]]))
      cs   = self.result.getType().getDomain().getCoordinateSystem()
      crv  = cs.fromReference(sets[s_idx].getSamples())
      dset = self.result.getDomainSet()
      lens = dset.getLengths()
      onImage = [[], []]
      ratio = self.caller.filereader.xlength/self.caller.filereader.gxlength
      for i in range(len(crv[0])):
        if ((crv[0][i] >= 0 and crv[0][i] < lens[0]) and (crv[1][i] >= 0 and crv[1][i] < lens[1])):
          crv[0][i] *= ratio
          crv[1][i] *= ratio
          onImage[0].append(crv[0][i])
          onImage[1].append(crv[1][i])
        else:
          crv[0][i] *= ratio
          crv[1][i] *= ratio
      
      if (len(onImage[0]) == 0):
        return
        
      lineLo = int(round(min(onImage[0])))
      lineHi = int(round(max(onImage[0])))
      elemLo = int(round(min(onImage[1])))
      elemHi = int(round(max(onImage[1])))
      
      insideCurve = [[], []]
      for j in range(elemHi - elemLo):
        for i in range(lineHi - lineLo):
           x = i + lineLo
           y = j + elemLo
           if DelaunayCustom.inside(crv, x, y):
             insideCurve[0].append(x)
             insideCurve[1].append(y)
      sc_list = self.caller.scatterDisplayList
      for i in range(len(sc_list)):
        sc_list[i].markScatterPointsByCurve(insideCurve[0], insideCurve[1], self.select_index)

      #-- copy to other result displays
      rd_list = self.caller.resultDisplayList
      self_idx = rd_list.index(self)
      for i in range(len(rd_list)):
        rd_list[i].rbbrj3d_s[self.select_index].removeLastBox()
        if (i != self_idx):
          rd_list[i].crvRef_s[self.select_index].setData(UnionSet([sets[s_idx]]))       
          rd_list[i].last_crvRef_s[self.select_index].setData(UnionSet([sets[s_idx]]))
              
  def windowClose(self, event):
    self.frame.dispose()
    self.caller.resultDisplayList.remove(self)
    runtime.gc()

    
#-------------------------------------------------------------------------------------    
    
class OperMenu(JComboBox):
  def __init__(self, pair):
    self.items = ["+","-","/","*"," "]
    JComboBox.__init__(self, self.items)
    self.pair = pair
    self.setSelectedIndex(self.items.index(self.pair.operator)) # do this before next line :)
    self.addActionListener(self)
       
  def actionPerformed(self, event):
    self.operator      = self.getSelectedItem()
    self.pair.operator = self.operator
    if (self.operator != " "):
      self.pair.rightEnable(1)
    else:
      self.pair.rightEnable(0)
      
  def toString(self):
    return self.operator
    

class OperationPair:
  def __init__(self, redInit, redSelectRef, greenInit, greenSelectRef, blueInit, blueSelectRef, clc):
    self.greenValue        = '%7.1f' % greenInit
    self.redValue          = '%7.1f' % redInit
    self.blueValue         = '%7.1f' % blueInit
    self.redValueDefault   = '%7.1f' % redInit
    self.greenValueDefault = '%7.1f' % greenInit
    self.blueValueDefault  = '%7.1f' % blueInit
    self.redTextField    = None
    self.greenTextField  = None
    self.blueTextField   = None
    self.redSelectRef    = redSelectRef
    self.greenSelectRef  = greenSelectRef
    self.blueSelectRef   = blueSelectRef
    self.clc             = clc
    
    self.mainPanel = JPanel()
    self.mainPanel.setLayout(FlowLayout())

    
    redTxt = self.redValue
    sp = JPanel()
    self.redTextField = JTextField(redTxt, actionPerformed=self.redOperand)
    self.redTextField.setBorder(LineBorder(Color.red, 2))
    sp.add(self.redTextField)
    self.mainPanel.add(sp)

    greenTxt = self.greenValue
    sp = JPanel()
    self.greenTextField = JTextField(greenTxt, actionPerformed=self.greenOperand)
    self.greenTextField.setBorder(LineBorder(Color.green, 2))
    sp.add(self.greenTextField)
    self.mainPanel.add(sp)

    blueTxt = self.blueValue
    sp = JPanel()
    self.blueTextField = JTextField(blueTxt, actionPerformed=self.blueOperand)
    self.blueTextField.setBorder(LineBorder(Color.blue, 2))
    sp.add(self.blueTextField)
    self.mainPanel.add(sp)

       
  def redOperand(self, event):
    txt = self.redTextField.getText()
    val = float(txt)
    nval = (self.clc.filereader.observationDomain.indexToValue(self.clc.filereader.observationDomain.valueToIndex([[val]])))[0][0]
    self.redValue = '%7.1f' % nval    
    type = self.redSelectRef.getData().getType()
    self.redSelectRef.setData(Real(type, nval))
    
  def setRed(self, txt):
    self.redTextField.setText(txt)
    
  def greenOperand(self, event):
    txt = self.greenTextField.getText()
    val = float(txt)
    nval = (self.clc.filereader.observationDomain.indexToValue(self.clc.filereader.observationDomain.valueToIndex([[val]])))[0][0]
    self.greenValue = '%7.1f' % nval
    type = self.greenSelectRef.getData().getType()
    self.greenSelectRef.setData(Real(type, nval))
  
  def setGreen(self, txt):
    self.greenTextField.setText(txt)

  def blueOperand(self, event):
    txt = self.blueTextField.getText()
    val = float(txt)
    nval = (self.clc.filereader.observationDomain.indexToValue(self.clc.filereader.observationDomain.valueToIndex([[val]])))[0][0]
    self.blueValue = '%7.1f' % nval
    type = self.blueSelectRef.getData().getType()
    self.blueSelectRef.setData(Real(type, nval))
                                                                                                                           
  def setBlue(self, txt):
    self.blueTextField.setText(txt)
    
  def getComponent(self):
    return self.mainPanel
    
  def evaluate(self):
    val = float(self.redTextField.getText())
    rdata = self.clc.getOperandData(val)
    type = rdata.getType()
    new_type = FunctionType(type.getDomain(), red_ch)
    dset = rdata.getDomainSet()
    values     = Helper.unpackFloats(rdata,false)
    self.rdata = FlatField(new_type, dset)
    self.rdata.setSamples(values, false)
    self.clc.red_minmax = [self.clc.filereader.obsMin[2], self.clc.filereader.obsMax[2]]

    val = float(self.greenTextField.getText())
    gdata = self.clc.getOperandData(val)
    type = gdata.getType()
    new_type = FunctionType(type.getDomain(), green_ch)
    dset = gdata.getDomainSet()
    values     = Helper.unpackFloats(gdata,false)
    self.gdata = FlatField(new_type, dset)
    self.gdata.setSamples(values, false)
    self.clc.grn_minmax = [self.clc.filereader.obsMin[2], self.clc.filereader.obsMax[2]]

    val = float(self.blueTextField.getText())
    bdata = self.clc.getOperandData(val)
    type = bdata.getType()
    new_type = FunctionType(type.getDomain(), blue_ch)
    dset = bdata.getDomainSet()
    values     = Helper.unpackFloats(bdata,false)
    self.bdata = FlatField(new_type, dset)
    self.bdata.setSamples(values, false)
    self.clc.blu_minmax = [self.clc.filereader.obsMin[2], self.clc.filereader.obsMax[2]]

    result = FieldImpl.combine([self.rdata, self.gdata, self.bdata])
      
    return result
  
  def evalatueFromFields(self, rdata, gdata, bdata):
    type = rdata.getType()
    new_type = FunctionType(type.getDomain(), red_ch)
    dset = rdata.getDomainSet()
    values     = Helper.unpackFloats(rdata,false)
    self.rdata = FlatField(new_type, dset)
    self.rdata.setSamples(values, false)

    type = gdata.getType()
    new_type = FunctionType(type.getDomain(), green_ch)
    dset = gdata.getDomainSet()
    values     = Helper.unpackFloats(gdata,false)
    self.gdata = FlatField(new_type, dset)
    self.gdata.setSamples(values, false)

    type = bdata.getType()
    new_type = FunctionType(type.getDomain(), blue_ch)
    dset = bdata.getDomainSet()
    values     = Helper.unpackFloats(bdata,false)
    self.bdata = FlatField(new_type, dset)
    self.bdata.setSamples(values, false)
    result = FieldImpl.combine([self.rdata, self.gdata, self.bdata])

    return result

  def lockout(self, lock):
    pass 
    
class OperationPairPullDown:
  def __init__(self, redInit, redSelectRef, greenInit, greenSelectRef, blueInit, blueSelectRef, clc):
    self.greenValue        = '%7.1f' % greenInit
    self.redValue          = '%7.1f' % redInit
    self.blueValue         = '%7.1f' % blueInit
    self.redValueDefault   = '%7.1f' % redInit
    self.greenValueDefault = '%7.1f' % greenInit
    self.blueValueDefault  = '%7.1f' % blueInit
    self.redTextField    = None
    self.greenTextField  = None
    self.blueTextField   = None
    self.redSelectRef    = redSelectRef
    self.greenSelectRef  = greenSelectRef
    self.blueSelectRef   = blueSelectRef
    self.clc             = clc
    
    self.mainPanel = JPanel()
    self.mainPanel.setLayout(FlowLayout())

    leftTxt = self.redValue
    sp = JPanel()
    
    band_numbers = []
    for i in xrange(len(self.clc.filereader.band_numbers)):
      band_numbers.append(self.clc.filereader.band_numbers[i])
    self.redTextField = JComboBox(band_numbers)
    
    class c1(java.awt.event.ActionListener):
      def __init__(self, filereader, comboBox, selectRef):
        self.filereader = filereader
        self.comboBox = comboBox
        self.selectRef = selectRef
        self.last_item = None
        self.lock_out  = 0
          
      def actionPerformed(self,event):
        item = self.comboBox.getSelectedItem()
        val = self.filereader.bandNameToValue(item)   
        nval = (self.filereader.observationDomain.indexToValue(self.filereader.observationDomain.valueToIndex([[val]])))[0][0]
        self.leftValue = '%5f' % nval    
        type = self.selectRef.getData().getType()
        if self.lock_out == 0:
          self.selectRef.setData(Real(type, nval))
          
    self.redListener = c1(self.clc.filereader, self.redTextField, self.redSelectRef)
    self.redTextField.addActionListener(self.redListener)
    self.redTextField.setBorder(LineBorder(Color.red, 2))
    sp.add(self.redTextField)
    self.mainPanel.add(sp)

    sp = JPanel()
    self.greenTextField = JComboBox(band_numbers)
    self.greenListener = c1(self.clc.filereader, self.greenTextField, self.greenSelectRef)
    self.greenTextField.addActionListener(self.greenListener)
    self.greenTextField.setBorder(LineBorder(Color.green, 2))
    sp.add(self.greenTextField)
    self.mainPanel.add(sp)

    sp = JPanel()
    self.blueTextField = JComboBox(band_numbers)
    self.blueListener = c1(self.clc.filereader, self.blueTextField, self.blueSelectRef)
    self.blueTextField.addActionListener(self.blueListener)
    self.blueTextField.setBorder(LineBorder(Color.blue, 2))
    sp.add(self.blueTextField)
    self.mainPanel.add(sp)

    
  def lockout(self, lock):
    self.redListener.lock_out = lock
    self.greenListener.lock_out = lock
    self.blueListener.lock_out = lock
  
  def getComponent(self):
    return self.mainPanel
    
  def evaluate(self):
    self.clc.redBandName = self.redTextField.getSelectedItem()
    val = self.clc.filereader.bandNameToValue(self.clc.redBandName)
    rdata = self.clc.getOperandData(val)
    type = rdata.getType()
    new_type = FunctionType(type.getDomain(), red_ch)
    dset = rdata.getDomainSet()
    values     = Helper.unpackFloats(rdata,false)
    self.rdata = FlatField(new_type, dset)
    self.rdata.setSamples(values, false)
    self.clc.red_minmax = [self.clc.filereader.obsMin[2], self.clc.filereader.obsMax[2]]
                                                                                                                           
    self.clc.grnBandName = self.greenTextField.getSelectedItem()
    val = self.clc.filereader.bandNameToValue(self.clc.grnBandName)
    gdata = self.clc.getOperandData(val)
    type = gdata.getType()
    new_type = FunctionType(type.getDomain(), green_ch)
    dset = gdata.getDomainSet()
    values     = Helper.unpackFloats(gdata,false)
    self.gdata = FlatField(new_type, dset)
    self.gdata.setSamples(values, false)
    self.clc.grn_minmax = [self.clc.filereader.obsMin[2], self.clc.filereader.obsMax[2]]
                                                                                                                           
    self.clc.bluBandName = self.blueTextField.getSelectedItem()
    val = self.clc.filereader.bandNameToValue(self.clc.bluBandName)
    bdata = self.clc.getOperandData(val)
    type = bdata.getType()
    new_type = FunctionType(type.getDomain(), blue_ch)
    dset = bdata.getDomainSet()
    values     = Helper.unpackFloats(bdata,false)
    self.bdata = FlatField(new_type, dset)
    self.bdata.setSamples(values, false)
    self.clc.blu_minmax = [self.clc.filereader.obsMin[2], self.clc.filereader.obsMax[2]]
                                                                                                                           
    result = FieldImpl.combine([self.rdata, self.gdata, self.bdata])
                                                                                                                           
    return result

  def evalatueFromFields(self, rdata, gdata, bdata):
    type = rdata.getType()
    new_type = FunctionType(type.getDomain(), red_ch)
    dset = rdata.getDomainSet()
    values     = Helper.unpackFloats(rdata,false)
    self.rdata = FlatField(new_type, dset)
    self.rdata.setSamples(values, false)
                                                                                                                               
    type = gdata.getType()
    new_type = FunctionType(type.getDomain(), green_ch)
    dset = gdata.getDomainSet()
    values     = Helper.unpackFloats(gdata,false)
    self.gdata = FlatField(new_type, dset)
    self.gdata.setSamples(values, false)
                                                                                                                               
    type = bdata.getType()
    new_type = FunctionType(type.getDomain(), blue_ch)
    dset = bdata.getDomainSet()
    values     = Helper.unpackFloats(bdata,false)
    self.bdata = FlatField(new_type, dset)
    self.bdata.setSamples(values, false)
    result = FieldImpl.combine([self.rdata, self.gdata, self.bdata])

    return result


      
  def setRed(self, txt):
    val = float(txt)
    self.redTextField.setSelectedItem(self.clc.filereader.bandValueToName(val))
    
  def setGreen(self, txt):
    val = float(txt)
    self.greenTextField.setSelectedItem(self.clc.filereader.bandValueToName(val))

  def setBlue(self, txt):
    val = float(txt)
    self.blueTextField.setSelectedItem(self.clc.filereader.bandValueToName(val))
      
  def toString(self):
    """
    if self.rghtOn == 1:
      return "("+self.leftTextField.getSelectedItem()+self.operator+self.rightTextField.getSelectedItem()+")"
    if self.rghtOn == 0:
      return "("+self.leftTextField.getSelectedItem()+")"
    """
    return "  "
