from visad import *  
from visad.bom import RubberBandLineRendererJ3D
import subs
from subs import * 
from visad.python.JPythonMethods import *
 
#---------------------------------------
false = 0
true  = 1
#---------------------------------------  


class TransectImage(CellImpl):
  displayLineType = None
  displayElemType = None
  displayAltType  = None
  dtt             = None
  count           = 0
  
  def __init__(self, image, dsp, xmap, ymap, callback, transectRef, color=None, ds_cs=None):
       
    #-------  display side coordinate system with rubberband box ------------- 
    rt   = image.getType().getDomain()
    cs   = rt.getCoordinateSystem()
    i_cs = IdentityCoordinateSystem(RealTupleType(makeType("Z")))
    cs   = CartesianProductCoordinateSystem([cs, i_cs])
    
    self.callback = callback
    self.transectRef = transectRef
    
    class TransformAdapter(CoordinateSystem):
      def __init__(self, cs, displayReference, smap_dspx, smap_dspy, ds_cs):
        self.cs               = cs
        self.ds_cs            = ds_cs
        self.displayReference = displayReference
        self.smap_dspx        = smap_dspx
        self.smap_dspy        = smap_dspy
        
        if ds_cs != None:
          ref     = cs.getReference()
          map_ref = ds_cs.getMapProjection().getReference()
        
        
        CoordinateSystem.__init__(self, displayReference, None)
        
        
      def toReference(self, values):
        new_values = self.cs.toReference(values)
        if self.ds_cs != None:
          new_values = self.ds_cs.toReference([new_values[1], new_values[0], new_values[2]])
        else:
          sox = getScale(self.smap_dspx)
          soy = getScale(self.smap_dspy)
          for i in range(len(new_values[0])):
            new_values[1][i] = new_values[1][i]*sox[0][0] + sox[0][1]
            new_values[0][i] = new_values[0][i]*soy[0][0] + soy[0][1]
          tmp = new_values[1]
          new_values[1] = new_values[0]
          new_values[0] = tmp
        
        return new_values
        
      def fromReference(self, values):
        #-scale values to lon/lat first
        if self.ds_cs == None:        
          sox = getScale(self.smap_dspx)
          soy = getScale(self.smap_dspy)
          for i in range(len(values[0])):
            values[0][i] = (values[0][i] - sox[0][1])/sox[0][0]
            values[1][i] = (values[1][i] - soy[0][1])/soy[0][0]     
          tmp       = values[1]
          values[1] = values[0]
          values[0] = tmp
          new_values = self.cs.fromReference(values)
        else:
          new_values = self.ds_cs.fromReference(values)
          new_values = self.cs.fromReference([new_values[1], new_values[0], new_values[2]])
        
        return new_values

      def equals(self, obj):
        return 1
        
    new_cs = TransformAdapter(cs, Display.DisplaySpatialCartesianTuple, xmap, ymap, ds_cs)
        
    TransectImage.displayLineType = DisplayRealType("displayTLine_%d" % TransectImage.count, true, 0.0, 10000.0, 0.0, None)
    TransectImage.displayElemType = DisplayRealType("displayTElem_%d" % TransectImage.count, true, 0.0, 10000.0, 0.0, None)
    TransectImage.displayAltType  = DisplayRealType("displayTAlt_%d" % TransectImage.count, true, -1.0, 1.0, 0.0, None)
    TransectImage.dtt = DisplayTupleType([TransectImage.displayLineType, TransectImage.displayElemType, TransectImage.displayAltType], new_cs)

    elemType = makeType("elem_%d" % TransectImage.count)
    lineType = makeType("line_%d" % TransectImage.count)
    self.boxRef = DataReferenceImpl("boxRef")
    self.noBox = Gridded2DSet(RealTupleType(lineType, elemType), None, 1)
    self.boxRef.setData(Gridded2DSet(RealTupleType(lineType, elemType), None, 1))
    dsp.addData("le", RealTuple(RealTupleType(lineType, elemType), [0, 0]), zlayer=-0.15)
    self.elemMap = ScalarMap(elemType, TransectImage.displayElemType)
    self.lineMap = ScalarMap(lineType, TransectImage.displayLineType)
    dset = image.getDomainSet()
    low  = dset.getLow()
    hi   = dset.getHi()
    self.elemMap.setRange(low[1], hi[1])
    self.lineMap.setRange(low[0], hi[0])
    dsp.addMap(self.elemMap)
    dsp.addMap(self.lineMap)
    self.rblrj3d = RubberBandLineRendererJ3D(lineType,elemType,0,0)
    constMap = None
    if color != None:
      constMap = makeColorMap(color)
    dsp.addReferences(self.rblrj3d, self.boxRef, constMap)
    
    self.addReference(self.boxRef)
    
    self.transectRef[0] = self.rblrj3d
    TransectImage.count += 1

    
  def doAction(self):
    samples = self.boxRef.getData().getSamples()
    if (samples != None):
      line = self.lineMap.scaleValues(samples[0])
      elem = self.elemMap.scaleValues(samples[1])
      
      self.callback(line,elem)
